package eaadapter.ea2ecore.postprocessings;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EClassImpl;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import eaadapter.ea2ecore.PostProcessingTemplate;
import eaadapter.ea2ecore.postprocessings.util.EcoreUtil;

public class AddGEastadlSuperClasses extends PostProcessingTemplate {
	
	private static final String GEASTADL_ECORE = "geastadl.ecore";
	private static final String MERGE_CONFIG = "merge.config";
	private static final String MODE = "mode";
	private static final String REPLACE = "replace";
	private static final String CLASS_MAPPING = "class.mapping";
	private static final String EA_MODEL_LAST_SGMT_PREFIX = "/eastadl";
	private static final String NONE = "none";
	private static final String TGT_ECLASS_NAME_SEPARATOR = ",";
	
	private File modelFile = null;
	
	private Map<EClass, Collection<String>> fMissingEClasses = new HashMap<EClass, Collection<String>>();

	public AddGEastadlSuperClasses (File modelFile) {
		this.modelFile = modelFile;
	}

	@Override
	public void execute() {
		
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
		
		IPath genModelPath = new Path(modelFile.getAbsolutePath());
		IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(
				genModelPath);
		IContainer folder = iFile.getParent();
		IResource geastadlEcoreFile = folder.findMember(GEASTADL_ECORE);
		
		if (geastadlEcoreFile == null || !geastadlEcoreFile.exists()) {
			return;
		}
		
		URI ecoreURI = URI.createPlatformResourceURI(geastadlEcoreFile.getFullPath().toString(), true);
		Resource geastadlEcoreRes = resourceSet.getResource(ecoreURI, true);
		
		ecoreURI = URI.createFileURI(modelFile.getAbsolutePath().toString());
		Resource targetEcore = Resource.Factory.Registry.INSTANCE.getFactory(ecoreURI).createResource(ecoreURI);
		targetEcore.getContents().add(model);
		
		resourceSet.getResources().add(targetEcore);
		
		List<EClass> gEClasses = EcoreUtil.getEAllEClasses(geastadlEcoreRes);
		String modelId = getModelId(targetEcore);
		for (EClass gEClass : gEClasses) {
			Iterable<EClass> tgtEClasses = getTargetEClasses(gEClass, modelId, targetEcore);
			for (EClass tgtEClass : tgtEClasses) {
				EAnnotation mergeConfig = gEClass.getEAnnotation(MERGE_CONFIG);
				if (mergeConfig != null) {
					String mode = mergeConfig.getDetails().get(MODE);
					if (REPLACE.equals(mode)) {
						tgtEClass.getESuperTypes().clear();
					}
				}
				tgtEClass.getESuperTypes().add(gEClass);
			}
		}
		handleMissingEClasses();	
	}
	
	private Collection<EClass> getTargetEClasses(EClass gEClass, String modelId, Resource targetModel) {
		Collection<String> tgtEClassNames = getTargetEClassNames(gEClass, modelId);
		Collection<EClass> tgtEClasses = new ArrayList<EClass>();
		if (isNoneMapping(tgtEClassNames)) {
			return Collections.<EClass> singleton(new NullEClass());
		}
		for (String tgtEClassName : tgtEClassNames) {
			EClass tgtEClass = EcoreUtil.findEClass(tgtEClassName, targetModel);
			if (tgtEClass != null) {
				tgtEClasses.add(tgtEClass);
			} else {
				if (!fMissingEClasses.containsKey(gEClass)) {
					fMissingEClasses.put(gEClass, new ArrayList<String>());
				}
				fMissingEClasses.get(gEClass).add(tgtEClassName);
			}
		}
		return tgtEClasses;
	}
	
	private Collection<String> getTargetEClassNames(EClass gEClass, String modelId) {
		Collection<String> mappedNames = getMappedTargetEClassNames(gEClass, modelId);
		if (!mappedNames.iterator().hasNext()) {
			mappedNames.add(gEClass.getName().substring(1));
		}
		return mappedNames;
	}
	
	private Collection<String> getMappedTargetEClassNames(EClass gEClass, String modelId) {
		Collection<String> mappedTgtNames = new ArrayList<String>();
		EAnnotation mapping = gEClass.getEAnnotation(CLASS_MAPPING);
		if (mapping != null) {
			String mappedTgtNameString = mapping.getDetails().get(modelId);
			if (mappedTgtNameString != null) {
				StringTokenizer tokenizer = new StringTokenizer(mappedTgtNameString, TGT_ECLASS_NAME_SEPARATOR);
				while (tokenizer.hasMoreTokens()) {
					mappedTgtNames.add(tokenizer.nextToken().trim());
				}
			}
		}
		return mappedTgtNames;
	}
	
	private boolean isNoneMapping(Collection<String> tgtEClassNames) {
		return tgtEClassNames.size() == 1 && NONE.equalsIgnoreCase(tgtEClassNames.iterator().next());
	}
	
	private String getModelId(Resource targetModel) {
		return getLastSegment(getNsURI(targetModel));
	}

	private String getNsURI(Resource targetModel) {
		EObject eObj = targetModel.getContents().get(0);
		if (!(eObj instanceof EPackage)) {
			throw new IllegalArgumentException("Could not identify target-model id. No root EPackage found for target-model!");
		}
		return ((EPackage) eObj).getNsURI();
	}

	private String getLastSegment(String nsURI) {
		int index = nsURI.lastIndexOf(EA_MODEL_LAST_SGMT_PREFIX);
		if (index == -1) {
			throw new IllegalArgumentException("Target-model is not an AUTOSAR meta-model!");
		}
		return nsURI.substring(index + 1);
	}
	
	private void handleMissingEClasses() {
		logMissingEClasses();
	}

	private void logMissingEClasses() {
		for (Entry<EClass, Collection<String>> missingEClasses : fMissingEClasses.entrySet()) {
			for (String tgtEClassName : missingEClasses.getValue()) {
				String logMsg = "The target EClass \"" + tgtEClassName + "\" for \"" + missingEClasses.getKey().getName() + "\" could not be found.";
				getLogger().info(logMsg);
			}
		}
	}
	
	private class NullEClass extends EClassImpl {

	}
}
