/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.ea2ecore.postprocessings;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Properties;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.codegen.ecore.genmodel.GenJDKLevel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.codegen.ecore.genmodel.GenResourceKind;
import org.eclipse.emf.codegen.ecore.genmodel.GenRuntimeVersion;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import eaadapter.common.util.Const;
import eaadapter.common.util.ModelConstants;
import eaadapter.ea2ecore.PostProcessingTemplate;

public class AdjustGenModel extends PostProcessingTemplate {

	private static final String GEASTADL_GENMODEL = ModelConstants.GEASTADL_GENMODEL;
    private static final String MODEL_CONFIG_PROPERTIES_NAME = ModelConstants.MODEL_CONFIG_PROPERTIES_NAME;
    private static final String EASTADL_STUB_GENMODEL = ModelConstants.EASTADL_STUB_GENMODEL;
    private static final String AUTOSAR_STUB_GENMODEL = ModelConstants.AUTOSAR_STUB_GENMODEL;
	
	private File modelFile;
	private String modelProject;
	private String metaModelType;
	
	/** GenModel-Properties */
	private String fileExtensions = ModelConstants.DEFAULT_EASTADL_FILE_EXTENSION;
	private String complianceLevel = null;
	private String copyrightTextFile = null;
	private boolean nonNLSMarkers = false;
	private String runtimeVersion = null;
	private String modelPluginClass = null;
	private String modelPluginVariables = null;
	private String editProject = null;
	private String editPluginClass = null;
	private String editPluginVariables = null;
	private String providerRootExtendsClass = null; 
	private boolean creationIcons = true;
	private String rootPackageContentTypeId = null;
	private String rootExtendsClass = null; 
	
	
	public static class UsedGenModel {

		private IFile file;

		public void setFile(IFile file) {
			this.file = file;
		}

		public List<GenPackage> load(ResourceSet resourceSet) {
			URI uri = URI.createPlatformResourceURI(file.getFullPath().toString(), true);
			Resource resource = resourceSet.getResource(uri, true);
			EObject eObject = resource.getContents().get(0);
			if (eObject instanceof GenModel) {
				GenModel genModel = (GenModel) eObject;
				return genModel.getAllGenPackagesWithClassifiers();
			} else {
				throw new IllegalArgumentException("Used GenModel file '" + file + "' must contain a GenModel instance.");
			}
		}
		
		public List<GenPackage> loadRootGenPackage(ResourceSet resourceSet) {
			URI uri = URI.createPlatformResourceURI(file.getFullPath().toString(), true);
			Resource resource = resourceSet.getResource(uri, true);
			EObject eObject = resource.getContents().get(0);
			if (eObject instanceof GenModel) {
				GenModel genModel = (GenModel) eObject;
				return genModel.getGenPackages();
			} else {
				throw new IllegalArgumentException("Used GenModel file '" + file + "' must contain a GenModel instance.");
			}
		}
	}
	
	/**
	 * Get the GENMODEL_PROPERTIES depends on the type of metamodel
	 * @return
	 */
	public String getGenModelProperties(){
		String GENMODEL_PROPERTIES = ModelConstants.EASTADL_GENMODEL_PROPERTIES;
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			GENMODEL_PROPERTIES = ModelConstants.SAFE_GENMODEL_PROPERTIES;
		}
		return GENMODEL_PROPERTIES;
	}
	
	/**
	 * Get the MODEL_CONFIG_PROPERTIES_PATH depends on the type of metamodel
	 * @return
	 */
	public String getModelConfigPropertiesPath(){
		String MODEL_CONFIG_PROPERTIES_PATH = Const.SOURCE_REVISION_ROOT_PATH + Const.SOURCE_REVISION_NAME_PREFIX; // "/sourcerevisions/model/eastadl-";
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			MODEL_CONFIG_PROPERTIES_PATH = Const.SOURCE_REVISION_ROOT_PATH + Const.SOURCE_REVISION_NAME_PREFIX_SAFE;  //"/sourcerevisions/model/safe-";
		}
		return MODEL_CONFIG_PROPERTIES_PATH;
	}
	
	
	public AdjustGenModel(File modelFile, String modelProject, String version, String metaModelType) {
		super();
		this.modelFile = modelFile;
		this.modelProject = modelProject;
		this.editProject = modelProject + ".edit";
		this.metaModelType = metaModelType;
		
		//load GenModel-Properties from properties file
		Properties props = new Properties();
		InputStream in;
		try {	
			//in = getClass().getResourceAsStream(GENMODEL_PROPERTIES);
			in = getClass().getResourceAsStream(getGenModelProperties());
			props.load(in);
			in.close();
		    this.fileExtensions = props.getProperty(ModelConstants.FILE_EXTENSIONS);
		    this.complianceLevel = props.getProperty(ModelConstants.COMPLIANCE_LEVEL);		    
		    this.nonNLSMarkers = Boolean.valueOf(props.getProperty(ModelConstants.NON_NLS_MARKERS));
		    this.runtimeVersion = props.getProperty(ModelConstants.RUNTIME_VERSION);
		    this.modelPluginClass = props.getProperty(ModelConstants.MODEL_PLUGIN_CLASS);
		    this.modelPluginVariables = props.getProperty(ModelConstants.MODEL_PLUGIN_VARIABLES);
		    this.editPluginClass = props.getProperty(ModelConstants.EDIT_PLUGIN_CLASS);
		    this.editPluginVariables = props.getProperty(ModelConstants.EDIT_PLUGIN_VARIABLES);
		    this.creationIcons = Boolean.valueOf(props.getProperty(ModelConstants.CREATION_ICONS));
		    this.providerRootExtendsClass = props.getProperty(ModelConstants.PROVIDER_ROOT_EXTENDS_CLASS);
		    this.rootExtendsClass = props.getProperty(ModelConstants.ROOT_EXTENDS_CLASS);
		    
		    if (version != null && version.length() > 0) {

				in = getClass().getResourceAsStream(getModelConfigPropertiesPath() 
						+ version + "/" + MODEL_CONFIG_PROPERTIES_NAME);
				props.load(in);
				in.close();
				this.rootPackageContentTypeId = props.getProperty(ModelConstants.ROOT_PACKAGE_CONTENT_TYPE_ID);
		    }
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void execute() {
		String fileName = modelFile.getPath().replace(".ecore", ".genmodel");
		File genModelFile = new File(fileName);
		
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
		URI uri;
		try {
			uri = URI.createFileURI(genModelFile.getAbsoluteFile().getCanonicalPath());
		} catch (IOException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			return;
		}
		Resource resource = resourceSet.getResource(uri, true);
		EObject eObject = resource.getContents().get(0);
		GenModel genModel;
		if (eObject instanceof GenModel) {
			genModel = (GenModel) eObject;
		} else {
			logger.error("GenModel file '" + genModelFile + "' must contain a GenModel instance.");
			return;
		}
		
		IPath genModelPath = new Path(modelFile.getAbsolutePath());
		IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(
				genModelPath);
		IContainer folder = iFile.getParent();
		
		List<GenPackage> before = genModel.getUsedGenPackages();
		for(GenPackage usedGenPackage : before){
			logger.info("1 Used genPackage = " + usedGenPackage.getPackageName());
		}
				
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_EASTADL)){			
			// Add used gen package: GEASTADL package
			IResource geastadlGenModel = folder.findMember(GEASTADL_GENMODEL);		
			if (geastadlGenModel != null && geastadlGenModel instanceof IFile
					&& geastadlGenModel.exists()) {
				UsedGenModel usedGenModel = new UsedGenModel();
				usedGenModel.setFile((IFile) geastadlGenModel);
				List<GenPackage> genPackageList = usedGenModel.load(resourceSet);
				genModel.getUsedGenPackages().addAll(genPackageList);
			}
		}
		else if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){				
			// Add used gen package: EASTADL STUB package
			IResource eastadl21GenModel = folder.findMember(EASTADL_STUB_GENMODEL);		
			if (eastadl21GenModel != null && eastadl21GenModel instanceof IFile
					&& eastadl21GenModel.exists()) {
				UsedGenModel usedEastadl21GenModel = new UsedGenModel();
				usedEastadl21GenModel.setFile((IFile) eastadl21GenModel);
				List<GenPackage> genPackageList = usedEastadl21GenModel.loadRootGenPackage(resourceSet);
				if(genPackageList!= null){
					for(GenPackage genPackage : genPackageList){
						logger.info("EAST-ADL STUB genPackage = " + genPackage.getPackageName());
					}
				}
				
				genModel.getUsedGenPackages().addAll(genPackageList);				
			}
			
			// Add used gen package: AUTOSAR STUB package
			IResource autosar41GenModel = folder.findMember(AUTOSAR_STUB_GENMODEL);		
			if (autosar41GenModel != null && autosar41GenModel instanceof IFile
					&& autosar41GenModel.exists()) {
				UsedGenModel usedAutosar41GenModel = new UsedGenModel();
				usedAutosar41GenModel.setFile((IFile) autosar41GenModel);
				List<GenPackage> genPackageList = usedAutosar41GenModel.loadRootGenPackage(resourceSet);
				if(genPackageList!= null){
					for(GenPackage genPackage : genPackageList){
						logger.info("AUTOSAR STUB genPackage = " + genPackage.getPackageName());
					}
				}
				genModel.getUsedGenPackages().addAll(genPackageList);
			}
		}
		
		try {
			resource.save(null);
		} catch (IOException e1) {
			logger.error("resource save error ");
		}

		IStatus status = genModel.validate();
		if (status.isOK()) {
			// Adjust GenModel settings
			adjustGenModel(genModel);

			// Adjust GenPackage settings
			for (GenPackage genPackage : genModel.getGenPackages()) {
				adjustGenPackage(genPackage);
			}

			try {
				resource.save(null);
			} catch (IOException e) {
				logger.error(e.getMessage());
			}
		} else {
			logger.error("Genmodel Status: " + status);
		}
	}

	protected void adjustGenPackage(GenPackage genPackage) {

		// Leave resource/resource factory generation enabled for root package only;
		// disable it for all sub packages
		if (genPackage.eContainer() != genPackage.getGenModel()) {
			genPackage.setResource(GenResourceKind.NONE_LITERAL);
			genPackage.setFileExtensions("");
		} 
		else {
				if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
					genPackage.setResource(GenResourceKind.XMI_LITERAL);
				}				
				genPackage.setFileExtensions(fileExtensions);
				if (this.rootPackageContentTypeId != null) {
					genPackage.setContentTypeIdentifier(this.rootPackageContentTypeId);
				}
		}

		// Package Suffixes section
		genPackage.setProviderPackageSuffix(ModelConstants.PROVIDER_PACKAGE_SUFFIX);

		// Adjust sub Package settings
		for (GenPackage subGenPackage : genPackage.getNestedGenPackages()) {
			adjustGenPackage(subGenPackage);
		}
	}

	protected void adjustGenModel(GenModel genModel) {
		
		// All section
		if (complianceLevel != null) {
			genModel.setComplianceLevel(GenJDKLevel.get(complianceLevel));
		}
		
		if (copyrightTextFile != null) {
			genModel.setCopyrightText(readTextFileContent(copyrightTextFile));
		}
		
		genModel.setNonNLSMarkers(nonNLSMarkers);
		
		if (runtimeVersion != null) {
			genModel.setRuntimeVersion(GenRuntimeVersion.get(runtimeVersion));
		}

		// Model & Model Class Defaults sections
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			genModel.setModelDirectory("/" + modelProject + ModelConstants.SRC_GEN_DIRECTORY);
		}
		else{
			genModel.setModelDirectory("/" + modelProject + ModelConstants.SRC_DIRECTORY);
		}
		
		if (modelPluginClass != null) {
			genModel.setModelPluginClass(
					genModel.getGenPackages().get(0).getPackageName() 
					+ "." + modelPluginClass);
		}
		genModel.setModelPluginID(modelProject);
		if (modelPluginVariables != null) {
			for (String variable : modelPluginVariables.split(",")) {
				if (variable.length() > 0) {
					genModel.getModelPluginVariables().add(variable);
				}
			}
		}

		// Edit section
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			genModel.setEditDirectory("/" + editProject + ModelConstants.SRC_GEN_DIRECTORY);
		}
		else{
			genModel.setEditDirectory("/" + editProject + ModelConstants.SRC_DIRECTORY);
		}
		
		
		if (editPluginClass != null) {
			genModel.setEditPluginClass(
					genModel.getGenPackages().get(0).getPackageName() 
					+ "." + editPluginClass);
		}
		genModel.setEditPluginID(editProject);
		if (editPluginVariables != null) {
			for (String variable : editPluginVariables.split(",")) {
				if (variable.length() > 0) {
					genModel.getEditPluginVariables().add(variable);
				}
			}
		}
		
		if (providerRootExtendsClass != null) {
			genModel.setProviderRootExtendsClass(providerRootExtendsClass);
		}
		
		if (rootExtendsClass != null) {
			genModel.setRootExtendsClass(rootExtendsClass);
		}
		
		genModel.setCreationSubmenus(true);
		genModel.setCreationIcons(creationIcons);
		
		genModel.setUpdateClasspath(true);

		// Disable Editor and Tests generation
		genModel.setEditorDirectory("");
		genModel.setTestsDirectory("");

		genModel.setCanGenerate(true);
	}
	
	private String readTextFileContent(String path) {
		StringBuilder content = new StringBuilder();
		BufferedReader input = null;
		try {
			input = new BufferedReader(new InputStreamReader(
					getClass().getResourceAsStream(path)));
			String line = null;
			while ((line = input.readLine()) != null) {
				content.append(line);
				content.append(System.getProperty("line.separator"));
			}
		} catch (Exception ex) {
			System.err.println("Unable to read content from file " + path + ": " + ex.getMessage());
		} finally {
			try {
				if (input != null) {
					input.close();
				}
			} catch (Exception ex) {
				// Do nothing
			}
		}

		return content.toString();
	}
}
