/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.ea2ecore.postprocessings;

import java.util.Iterator;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreSwitch;
import org.eclipse.emf.ecore.util.ExtendedMetaData;

import eaadapter.common.util.ModelConstants;
import eaadapter.common.util.ModelUtil;
import eaadapter.ea2ecore.PostProcessingTemplate;

public class CreateExtendedMetaData extends PostProcessingTemplate {
	
	EcoreSwitch<Object> eSwitch = new EcoreSwitch<Object>() {
		@Override
		public Object caseEClass(EClass object) {
			return createExtendedMetaData(object);
		}

		@Override
		public Object caseEPackage(EPackage object) {
			return createExtendedMetaData(object);
		}

		@Override
		public Object caseEAttribute(EAttribute object) {
			return createExtendedMetaData(object);
		}
		
		@Override
		public Object caseEReference(EReference object) {
			return createExtendedMetaData(object);
		}
	};
	
	private String version = null;
	
	public CreateExtendedMetaData(String version) {
		super();
		this.version = version;
	}
	
	private Object createExtendedMetaData(EAttribute eAttribute) {
		EAnnotation taggedValuesAnnotation = eAttribute.getEAnnotation(
				ModelConstants.ANNOTATION_TAGGED_VALUES);
		
		if (taggedValuesAnnotation != null && taggedValuesAnnotation.getDetails() != null) {
			String name = ModelUtil.getTaggedValue(eAttribute, 
					ModelConstants.TAGGED_VALUE_XML_NAME);
			if (name != null) {
				ExtendedMetaData.INSTANCE.setName(eAttribute, name);
			}
			
			boolean isAttribute = "true".equalsIgnoreCase(
					taggedValuesAnnotation.getDetails().get(
							ModelConstants.TAGGED_VALUE_XML_ATTRIBUTE));
			
			if (!isAttribute && isComplexTypeWithSimpleContent(eAttribute.getEContainingClass())) { 
				ExtendedMetaData.INSTANCE.setFeatureKind(eAttribute, ExtendedMetaData.SIMPLE_FEATURE);
			} else {
				ExtendedMetaData.INSTANCE.setFeatureKind(eAttribute, isAttribute ? ExtendedMetaData.ATTRIBUTE_FEATURE
						: ExtendedMetaData.ELEMENT_FEATURE);
			}
	
			if (!isAttribute) {
				ExtendedMetaData.INSTANCE.setNamespace(eAttribute, getXMLNamespace());
			}
		}
		
		return eAttribute;
	}
	
	/**
	 * Creates ExtendedMetaData for an EPackage and iterates through 
	 * all subpackages and classifier
	 * 
	 * @param ePackage
	 * @return ePackage
	 */
	private Object createExtendedMetaData(EPackage ePackage) {
		
		ExtendedMetaData.INSTANCE.setQualified(ePackage, false);
		
		Iterator<EClassifier> iterCls = ePackage.getEClassifiers().iterator();
		while (iterCls.hasNext()) {
			eSwitch.doSwitch(iterCls.next());
		}
		
		//iterate through subpackages
		Iterator<EPackage> iter = ePackage.getESubpackages().iterator();
		while (iter.hasNext()) {
			eSwitch.doSwitch(iter.next());
		}
		
		return ePackage;
	}
	
	/**
	 * Creates ExtendedMetaData for an EReference
	 * 
	 * @param eRef
	 * @return
	 */
	private Object createExtendedMetaData(EReference eRef) {
		boolean isRoleElement = "true".equalsIgnoreCase(ModelUtil
				.getTaggedValue(eRef, ModelConstants.TAGGED_VALUE_XML_ROLE_ELEMENT));
		boolean isTypeElement = "true".equalsIgnoreCase(ModelUtil.
				getTaggedValue(eRef, ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT));
		boolean isRoleWrapperElement = "true".equalsIgnoreCase(ModelUtil.
				getTaggedValue(eRef, 
						ModelConstants.TAGGED_VALUE_XML_ROLE_WRAPPER_ELEMENT));
		boolean isTypeWrapperElement = "true".equalsIgnoreCase(ModelUtil
				.getTaggedValue(eRef,
						ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT));

		// There are some cases where roleWrapper=true and the upperMultiplicity is 1. Therefore the
		// upperMultiplicity is not checked when determining the xml.name.
		String name;
		if (isRoleWrapperElement) {
			name = ModelUtil.getTaggedValue(eRef, 
					ModelConstants.TAGGED_VALUE_XML_NAME_PLURAL);
		} else {
			name = ModelUtil.getTaggedValue(eRef, 
					ModelConstants.TAGGED_VALUE_XML_NAME);
		}

		// 1000 ! This case is not allowed by the spec, but occurs
		if (isRoleWrapperElement && !isRoleElement && !isTypeElement 
				&& !isTypeWrapperElement) {
			name = ModelUtil.getTaggedValue(eRef, 
					ModelConstants.TAGGED_VALUE_XML_NAME);
		}

		ExtendedMetaData.INSTANCE.setName(eRef, name);
		ExtendedMetaData.INSTANCE.setFeatureKind(eRef, 
				ExtendedMetaData.ELEMENT_FEATURE);
		ExtendedMetaData.INSTANCE.setNamespace(eRef, getXMLNamespace());
		
		return eRef;
	}
	
	/**
	 * Creates ExtendedMetaData for an EClass and iterates through all its features
	 * 
	 * @param eClass
	 * @return eClass
	 */
	private Object createExtendedMetaData(EClass eClass) {

		ExtendedMetaData.INSTANCE.setName(eClass, ModelUtil.getTaggedValue(eClass, 
						ModelConstants.TAGGED_VALUE_XML_NAME));

		if (ExtendedMetaData.INSTANCE.getContentKind(eClass) 
				!= ExtendedMetaData.UNSPECIFIED_CONTENT) {
			// content kind is already set, so do not overwrite
			return eClass;
		}
		
		int kind = ExtendedMetaData.ELEMENT_ONLY_CONTENT;
		if (isComplexTypeWithSimpleContent(eClass)) {
			kind = ExtendedMetaData.SIMPLE_CONTENT;
		}
		ExtendedMetaData.INSTANCE.setContentKind(eClass, kind);
		
		
		//iterate through features
		Iterator<EStructuralFeature> iter = eClass.getEStructuralFeatures().iterator();
		while (iter.hasNext()) {
			eSwitch.doSwitch(iter.next());	
		}
		
		return eClass;
	}
	
	@Override
	public void execute() {
		createExtendedMetaData(model);
	}
	
	/**
	 * Returns true if currentClass satisfies the criteria of APRXML0024.
	 * 
	 * @param currentClass
	 * @return As specified aboved.
	 */
	protected boolean isComplexTypeWithSimpleContent(EClass currentClass) {
		boolean isComplexTypeWithSimpleContent = false;

		if (currentClass != null) {
			// If the class is not abstract and it contains exactly one (derived
			// or owned) property that is not mapped
			// to an xml.attribute
			// and this property is not represented by any XML elements (tagged
			// values xml.*Element=false)
			// then a xsd:complexType with simpleContent is generated.
			// The simpleContent contains the data of the property which is not
			// mapped to an xml.attribute.
			if (!currentClass.isAbstract()) {
				int propertyNotAttributeCount = 0;
				EStructuralFeature candidateAttribute = null;
				for (EStructuralFeature property : currentClass
						.getEAllStructuralFeatures()) {
					propertyNotAttributeCount++;
					boolean isRoleElement = "true"
							.equalsIgnoreCase(ModelUtil.getTaggedValue(property,
											ModelConstants.TAGGED_VALUE_XML_ROLE_ELEMENT));
					boolean isTypeElement = "true"
							.equalsIgnoreCase(ModelUtil.getTaggedValue(property,
											ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT));
					boolean isRoleWrapperElement = "true"
							.equalsIgnoreCase(ModelUtil.getTaggedValue(property,
											ModelConstants.TAGGED_VALUE_XML_ROLE_WRAPPER_ELEMENT));
					boolean isTypeWrapperElement = "true"
							.equalsIgnoreCase(ModelUtil.getTaggedValue(property,
											ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT));
					if (!isRoleElement && !isTypeElement
							&& !isRoleWrapperElement && !isTypeWrapperElement) {
						// type of property that is not mapped to xml attribute
						// is <<primitive>> or
						// <<enumeration>> AND
						// upper multiplicity of property that is not mapped to
						// xml attribute is 1
						if (property.getUpperBound() != 1) {
							continue;
						}
						// type of property that is not mapped to xml attribute
						// is <<primitive>> or
						// <<enumeration>> AND
						// EEnum extends EDataType ...
						if (property.getEType() instanceof EDataType) {
							candidateAttribute = property;
						}
					}
					if (propertyNotAttributeCount > 1) {
						return false;
					}
				}
				if (propertyNotAttributeCount == 1
						&& candidateAttribute != null) {
					return true;
				}
			}
		}

		return isComplexTypeWithSimpleContent;
	}
	
	protected String getXMLNamespace() {
		StringBuilder namespace = new StringBuilder(ModelConstants.DEFAULT_NSURI);
		if (version != null) {
			namespace.append("/");
			namespace.append(version);
		}
		return namespace.toString();
	}

}
