/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.ea2ecore.postprocessings;

import java.util.Iterator;
import java.util.Stack;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.ETypedElement;
import org.eclipse.emf.ecore.util.EcoreSwitch;

import eaadapter.common.util.ModelConstants;
import eaadapter.common.util.ModelUtil;
import eaadapter.ea2ecore.PostProcessingTemplate;
import eaadapter.ea2ecore.postprocessings.util.SchemaUtil;

public class SetTaggedValuesAndAssociatedAttributes extends
		PostProcessingTemplate {
	
	private Stack<String> containerNsPrefixStack = new Stack<String>();

	private String containerNsPrefix = ModelConstants.DEFAULT_NSPREFIX;

	private Stack<String> containerNsUriStack = new Stack<String>();

	private String defaultNsUri = null;
	private String containerNsUri = ModelConstants.DEFAULT_NSURI;
	private String version = null;
	
	public SetTaggedValuesAndAssociatedAttributes(String version) {
		
		this.defaultNsUri = ModelConstants.DEFAULT_NSURI;
		this.version = version;
		
		if (version != null && version.length() > 0) {
			this.defaultNsUri = this.defaultNsUri + "/" + version;
		}
		this.containerNsUri = this.defaultNsUri;
	}
	
	private EcoreSwitch<Object> eSwitch = new EcoreSwitch<Object>() {
		public Object caseEPackage(EPackage object) {
			return configure(object);
		}

		public Object caseEClass(EClass object) {
			return configure(object);
		}

		public Object caseEStructuralFeature(EStructuralFeature object) {
			return configure(object);
		}

		public Object caseEDataType(EDataType object) {
			return configure(object);
		}

		public Object caseEEnumLiteral(EEnumLiteral object) {
			return configure(object);
		}

		public Object caseEEnum(EEnum object) {
			return configure(object);
		}
	};
	
	/**
	 * Sets the following tagged values of an ENamedElement
	 * 
	 * <ul>
	 * <li>xml.name</li>
	 * <li>xml.namePlural</li>
	 * <li>xml.nsPrefix</li>
	 * <li>xml.nsUri</li>
	 * 
	 * @param element
	 */
	protected void configureName(ENamedElement element) {
		
		String suffixSingualr = "";
		String suffixPlural = "S";
		
		// check if namedElement is an EReference
		if (element instanceof EReference) {
			EReference ref = (EReference) element;
			// check if reference is not a composition
			if (!ref.isContainment()) {
				// check if reference is a normal reference or a type reference
				if (ModelConstants.STEREOTYPE_IS_OF_TYPE
						.equalsIgnoreCase(ModelUtil.getStereotype(ref))) {
					// is of type reference found
					suffixSingualr = ModelConstants.SUFFIX_SINGULAR_TYPE_REF;
					suffixPlural = ModelConstants.SUFFIX_PLURAL_TYPE_REF;
				} else {
					// normal ref
					suffixSingualr = ModelConstants.SUFFIX_SINGULAR_REF;
					suffixPlural = ModelConstants.SUFFIX_PLURAL_REF;
				}
			}
		}
		
		//set tagged values
		ModelUtil.addDefaultTaggedValue(element, ModelConstants.TAGGED_VALUE_XML_NAME, 
				SchemaUtil.buildXmlName(element.getName()) + suffixSingualr);
		
		ModelUtil.addDefaultTaggedValue(element, ModelConstants.TAGGED_VALUE_XML_NAME_PLURAL, 
				SchemaUtil.buildXmlName(element.getName()) + suffixPlural);
		
		ModelUtil.addDefaultTaggedValue(element, ModelConstants.TAGGED_VALUE_XML_NS_PREFIX, 
				containerNsPrefix);
		
		if(metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			this.defaultNsUri = ModelConstants.DEFAULT_SAFE_NSURI;
			if (version != null && version.length() > 0) {
				this.defaultNsUri = this.defaultNsUri + "/" + version;
			}
			this.containerNsUri = this.defaultNsUri;
		}
				
		ModelUtil.addDefaultTaggedValue(element, ModelConstants.TAGGED_VALUE_XML_NS_URI, 
				containerNsUri);
		
        ModelUtil.setEAAdapterAnnotation(element, 
        		ModelConstants.EAADAPTER_METAMODEL_TYPE, metaModelType);
		
	}
	
	/**
	 * Pushes current namespace data onto the stack and sets 
	 * namespace data of the element as current namespace data
	 * 
	 * @param element
	 */
	protected void pushNs(ENamedElement element) {
		String nsPrefix = ModelUtil.getTaggedValue(element, 
				ModelConstants.TAGGED_VALUE_XML_NS_PREFIX);
		String nsUri = ModelUtil.getTaggedValue(element, 
				ModelConstants.TAGGED_VALUE_XML_NS_URI);
		
		containerNsPrefixStack.push(containerNsPrefix);
		containerNsPrefix = nsPrefix;
		
		containerNsUriStack.push(containerNsUri);
		containerNsUri = nsUri;
	}
	
	/**
	 * Pops namespace data from stack and sets them as
	 * current namespace data. If stack is empty then the
	 * default namespace data will be used.
	 */
	protected void popNs() {
		if (containerNsPrefixStack.isEmpty()) {
			containerNsPrefix = ModelConstants.DEFAULT_NSPREFIX;
		} else {
			containerNsPrefix = containerNsPrefixStack.pop();
		}
		
		if (containerNsUriStack.isEmpty()) {
			containerNsUri = defaultNsUri;
		} else {
			containerNsUri = containerNsUriStack.pop();
		}
	}
	
	/**
	 * Configures tagged values of a package.
	 * 
	 * @param ePackage
	 * @return ePackage
	 */
	private Object configure(EPackage ePackage) {
		configureName(ePackage);
		pushNs(ePackage);
		
		
		Iterator<EClassifier> clsIter = ePackage.getEClassifiers().iterator();
		while (clsIter.hasNext()) {
			eSwitch.doSwitch(clsIter.next());
		}
		
		Iterator<EPackage> subPckIter = ePackage.getESubpackages().iterator();
		while (subPckIter.hasNext()) {
			eSwitch.doSwitch(subPckIter.next());
		}
		
		popNs();
		return ePackage;
	}
	
	/**
	 * Configures tagged values of a EEnum
	 * 
	 * @param eEnum
	 * @return eEnum
	 */
	private Object configure(EEnum eEnum) {
		configureName(eEnum);
		pushNs(eEnum);
		
		Iterator<EEnumLiteral> iter = eEnum.getELiterals().iterator();
		while (iter.hasNext()) {
			eSwitch.doSwitch(iter.next());
		}
		
		popNs();
		return eEnum;
	}
	
	/**
	 * Configures tagged values of a EEnumLiteral
	 * 
	 * @param literal
	 * @return literal
	 */
	private Object configure(EEnumLiteral literal) {
		configureName(literal);
		return literal;
	}
	
	/**
	 * Configures tagged values of a primitive Datatype
	 * 
	 * @param dataType
	 * @return
	 */
	private Object configure(EDataType dataType) {
		configureName(dataType);
		return dataType;
	}
	
	/**
	 * Configures tagged values of a EClass
	 * 
	 * @param eClass
	 * @return eClass
	 */
	private Object configure(EClass eClass) {
		configureName(eClass);
		pushNs(eClass);
		
		Iterator<EStructuralFeature> iter = null;
		
		if (ModelUtil.getStereotype(eClass).equalsIgnoreCase(
				ModelConstants.STEREOTYPE_ATPMIXED)
				|| ModelUtil.getStereotype(eClass).equalsIgnoreCase(
						ModelConstants.STEREOTYPE_ATPMIXEDSTRING)) {

			iter = eClass.getEStructuralFeatures().iterator();
			while (iter.hasNext()) {
				EStructuralFeature feature = iter.next();

				feature.setUpperBound(ETypedElement.UNBOUNDED_MULTIPLICITY);
				feature.setLowerBound(0);

				ModelUtil.addDefaultTaggedValue(feature, 
						ModelConstants.TAGGED_VALUE_XML_ROLE_WRAPPER_ELEMENT, 
						ModelConstants.FALSE_STRING);

				ModelUtil.addDefaultTaggedValue(feature, 
						ModelConstants.TAGGED_VALUE_XML_ROLE_ELEMENT, 
						ModelConstants.TRUE_STRING);

				EClassifier eClassifier = feature.getEType();

				if ((eClassifier instanceof EClass)
						&& hasConcreteSubclasses((EClass) eClassifier)) {
					// subclasses exists
					ModelUtil.addDefaultTaggedValue(feature, 
							ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT, 
							ModelConstants.FALSE_STRING);

					ModelUtil.addDefaultTaggedValue(feature, 
							ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT, 
							ModelConstants.TRUE_STRING);

				} else {
					// subclasses exists
					ModelUtil.addDefaultTaggedValue(feature, 
							ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT, 
							ModelConstants.FALSE_STRING);
					ModelUtil.addDefaultTaggedValue(feature, 
							ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT, 
							ModelConstants.FALSE_STRING);
				}
			}
		}
		
		iter = eClass.getEStructuralFeatures().iterator();
		while (iter.hasNext()) {
			eSwitch.doSwitch(iter.next());
		}
		
		popNs();
		return eClass;
	}
	
	/**
	 * configures how a structural features (which is an Ecore EReference or an
	 * EAttribute or a UML Property) is mapped to XML. The following tagged
	 * values are set:
	 * <ul>
	 * <li>xml.roleWrapperElement</li>
	 * <li>xml.roleElement</li>
	 * <li>xml.typeWrapperElement</li>
	 * <li>xml.typeElement</li>
	 * </ul>
	 * 
	 * @param eStructuralFeature
	 */
	protected void configureElementMapping(EStructuralFeature eStructuralFeature) {

			boolean isContainment = eStructuralFeature instanceof EAttribute
					|| ((EReference) eStructuralFeature).isContainment();

			// role
			String roleWrapperElement;
			String roleElement;
			if (eStructuralFeature.getUpperBound() > 1
					|| eStructuralFeature.getUpperBound() == ETypedElement.UNBOUNDED_MULTIPLICITY) {
				roleWrapperElement = ModelConstants.BOOLEAN_TRUE;

				// for references, a role wrapper is default (the -REF element):
				roleElement = Boolean.toString(!isContainment);
			} else {
				roleWrapperElement = ModelConstants.BOOLEAN_FALSE;
				roleElement = ModelConstants.BOOLEAN_TRUE;
			}

			ModelUtil.addDefaultTaggedValue(eStructuralFeature,
					ModelConstants.TAGGED_VALUE_XML_ROLE_ELEMENT, roleElement);

			ModelUtil.addDefaultTaggedValue(eStructuralFeature,
					ModelConstants.TAGGED_VALUE_XML_ROLE_WRAPPER_ELEMENT,
					roleWrapperElement);

			// type:
			String typeElement;
			if (isContainment
					&& eStructuralFeature.getEType() instanceof EClass) {

				if (hasConcreteSubclasses((EClass) eStructuralFeature.getEType())
						|| eStructuralFeature.getUpperBound() > 1
						|| eStructuralFeature.getUpperBound() == ETypedElement.UNBOUNDED_MULTIPLICITY
						|| "true".equalsIgnoreCase(ModelUtil.getTaggedValue(
								eStructuralFeature.getEType(),
								ModelConstants.TAGGED_VALUE_EXTENSION_POINT))) {
					typeElement = ModelConstants.BOOLEAN_TRUE;
				} else {
					typeElement = ModelConstants.BOOLEAN_FALSE;
				}
			} else {
				// dirty hack, refinement required for primitive datatypes, etc.
				typeElement = ModelConstants.BOOLEAN_FALSE;
			}

			ModelUtil.addDefaultTaggedValue(eStructuralFeature,
					ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT, typeElement);

			ModelUtil.addDefaultTaggedValue(eStructuralFeature,
					ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT, 
					ModelConstants.BOOLEAN_FALSE);
	}
	
	/**
	 * Configures tagged values of a EStructuralFeature
	 * 
	 * @param feature
	 * @return feature
	 */
	protected Object configure(EStructuralFeature feature) {
		configureName(feature);
		pushNs(feature);
		configureElementMapping(feature);
		popNs();
		return feature;
	}

	@Override
	public void execute() {
		configure(model);
	}
	

    /**
     * Checks if a EClass has concrete sub classes.
     * 
     * @param eClass super class
     * @return true if eClass has sub classes
     */
    private boolean hasConcreteSubclasses(EClass eClass) {
        Iterator<EObject> iterator = model.eAllContents();
        while (iterator.hasNext()) {
        	EObject element = iterator.next();
            if (element instanceof EClass) {
                EClass potentialSubclass = (EClass) element;
                if (!potentialSubclass.isAbstract() && potentialSubclass.getESuperTypes().contains(eClass)) {
                    return true;
                }
            }
        }
        return false;
    }

}
