/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */

package eaadapter.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.BeforeClass;
import org.junit.Test;

import eaadapter.EAAttribute;
import eaadapter.EAConnector;
import eaadapter.EAElement;
import eaadapter.EARepository;
import eaadapter.EaadapterFactory;
import eaadapter.datatypes.MetaModelType;
import eaadapter.datatypes.T_ConnectorAggregation;
import eaadapter.util.Util;

public class TestLoadEAModel {
	
	private static EARepository eaModel;
	private static File eaModelFile;
	private static final String filename = "resources/testmodel.eap";
	
	@BeforeClass
	public static void initializeEAModel() throws Exception {
		eaModelFile = new File(filename);
		assertTrue("testmodel.eap exists", eaModelFile.exists());
		eaModel = EaadapterFactory.eINSTANCE.createEARepository();
		assertNotNull("eaModel must not be null", eaModel);
		eaModel.setFile(eaModelFile);
		eaModel.setTypeOfMetaModel(MetaModelType.EASTADL);
		eaModel.load();
		
		assertTrue("Model contains more than 1 element", eaModel.getModels().size() > 0);	
	}

	/**
	 * This method tests the Element 'EAST-ADL/Structure/FunctionModeling/FunctionType'
	 */
	@Test
	public void testFunctionType() {
		EAElement FunctionType = Util.findEAElementByName("FunctionType", eaModel);
		assertNotNull("Could not find FunctionType", FunctionType);
		assertTrue(FunctionType.isIsAbstract());
		assertTrue(FunctionType.getMetaType().equals("Class"));
		assertEquals(FunctionType.getPackage().getName(), "FunctionModeling");
		
		assertTrue("FunctionType only has 1 Attribute", FunctionType.getAttributes().size() == 1);
		EAAttribute isElementary = Util.getEAAttributeForEAElement("isElementary", FunctionType, eaModel);
		assertNotNull(isElementary);
		assertEquals(isElementary.getType(), "Boolean");
		
		EAConnector connectorPort = Util.getEAConnectorByRolenameForEAElement("port", FunctionType, eaModel);
		assertNotNull("Could not find Connector 'port'", connectorPort);
		assertEquals("Connector must be a containment", connectorPort.getClientEnd().getAggregation(), T_ConnectorAggregation.COMPOSITE);
		assertEquals(connectorPort.getClientEnd().getCardinality(), "0..1");
		assertEquals(connectorPort.getSupplierEnd().getCardinality(), "*");	
		
		int numberOfSuperTypes = 0;
		for (EAConnector connector : FunctionType.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("FunctionType must have 2 supertypes", 2, numberOfSuperTypes);
	}
	
	@Test
	public void testFunctionalDevice() {
		EAElement FunctionalDevice = Util.findEAElementByName("FunctionalDevice", eaModel);
		assertNotNull("Could not find FunctionalDevice", FunctionalDevice);
		assertTrue(FunctionalDevice.getMetaType().equals("Class"));
		assertEquals(FunctionalDevice.getPackage().getName(), "FunctionModeling");
		FunctionalDevice.getConnectors();
	}
	
	@Test
	public void testFunctionalAllocation() {
		EAElement FunctionAllocation = Util.findEAElementByName("FunctionAllocation", eaModel);
		assertNotNull("Could not find FunctionAllocation", FunctionAllocation);
		assertTrue(FunctionAllocation.getMetaType().equals("Class"));
		assertEquals(FunctionAllocation.getPackage().getName(), "FunctionModeling");
		FunctionAllocation.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : FunctionAllocation.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("FunctionAllocation must have 1 supertypes", 1, numberOfSuperTypes);
	}
	
	@Test
	public void testAllocation() {
		EAElement Allocation = Util.findEAElementByName("Allocation", eaModel);
		assertNotNull("Could not find Allocation", Allocation);
		assertTrue(Allocation.getMetaType().equals("Class"));
		assertEquals(Allocation.getPackage().getName(), "FunctionModeling");
		Allocation.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Allocation.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Allocation must have 1 supertypes", 1, numberOfSuperTypes);
	}
	
	@Test
	public void testOperation() {
		EAElement Operation = Util.findEAElementByName("Operation", eaModel);
		assertNotNull("Could not find Operation", Operation);
		assertTrue(Operation.getMetaType().equals("Class"));
		assertTrue(Operation.getStatus().equals("Proposed"));
		assertEquals(Operation.getPackage().getName(), "FunctionModeling");
		Operation.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Operation.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Operation must have 1 supertypes", 1, numberOfSuperTypes);
	}
	@Test
	public void testLocalDeviceManager() {
		EAElement LocalDeviceManager = Util.findEAElementByName("LocalDeviceManager", eaModel);
		assertNotNull("Could not find LoccalDeviceManager", LocalDeviceManager);
		assertTrue(LocalDeviceManager.getMetaType().equals("Class"));
		assertTrue(LocalDeviceManager.getStatus().equals("Proposed"));
		assertEquals(LocalDeviceManager.getPackage().getName(), "FunctionModeling");
		LocalDeviceManager.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : LocalDeviceManager.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("LocalDeviceManager must have 1 supertypes", 1, numberOfSuperTypes);
	}

	@Test
	public void testvehicleLevel() {
		EAElement VehicleLevel = Util.findEAElementByName("VehicleLevel", eaModel);
		assertNotNull("Could not find vehicleLevel", VehicleLevel);
		assertTrue(VehicleLevel.getMetaType().equals("Class"));
		assertTrue(VehicleLevel.getStatus().equals("Proposed"));
		assertEquals(VehicleLevel.getPackage().getName(), "SystemModeling");
		VehicleLevel.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : VehicleLevel.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("VehicleLevel must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue(VehicleLevel.getStereotype().equals("atpStructureElement"));
		assertTrue(VehicleLevel.getStereotypeEx().equals("atpStructureElement"));
	}

	@Test
	public void testSystemModel() {
		EAElement SystemModel = Util.findEAElementByName("SystemModel", eaModel);
		assertNotNull("Could not find SystemModel", SystemModel);
		assertTrue(SystemModel.getMetaType().equals("Class"));
		assertTrue(SystemModel.getStatus().equals("Proposed"));
		assertEquals(SystemModel.getPackage().getName(), "SystemModeling");
		SystemModel.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : SystemModel.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("SystemModel must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue(SystemModel.getStereotype().equals("atpStructureElement"));
		assertTrue(SystemModel.getStereotypeEx().equals("atpStructureElement"));
	}
	
	@Test
	public void testAnalysisLevel() {
		EAElement AnalysisLevel = Util.findEAElementByName("AnalysisLevel", eaModel);
		assertNotNull("Could not find AnalysisLevel", AnalysisLevel);
		assertTrue(AnalysisLevel.getMetaType().equals("Class"));
		assertTrue(AnalysisLevel.getStatus().equals("Proposed"));
		assertEquals(AnalysisLevel.getPackage().getName(), "SystemModeling");
		AnalysisLevel.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : AnalysisLevel.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("AnalysisLevel must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue(AnalysisLevel.getStereotype().equals("atpStructureElement"));
		assertTrue(AnalysisLevel.getStereotypeEx().equals("atpStructureElement"));
	}
	
	@Test
	public void testDesignLevel() {
		EAElement DesignLevel = Util.findEAElementByName("DesignLevel", eaModel);
		assertNotNull("Could not find DesignLevel", DesignLevel);
		assertTrue(DesignLevel.getMetaType().equals("Class"));
		assertTrue(DesignLevel.getStatus().equals("Proposed"));
		assertEquals(DesignLevel.getPackage().getName(), "SystemModeling");
		DesignLevel.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : DesignLevel.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("DesignLevel must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue(DesignLevel.getStereotype().equals("atpStructureElement"));
		assertTrue(DesignLevel.getStereotypeEx().equals("atpStructureElement"));
	}
	
	@Test
	public void testImplementationLevel() {
		EAElement ImplementationLevel = Util.findEAElementByName("ImplementationLevel", eaModel);
		assertNotNull("Could not find ImplementationLevel", ImplementationLevel);
		assertTrue(ImplementationLevel.getMetaType().equals("Class"));
		assertTrue(ImplementationLevel.getStatus().equals("Proposed"));
		assertEquals(ImplementationLevel.getPackage().getName(), "SystemModeling");
		ImplementationLevel.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : ImplementationLevel.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("ImplementationLevel must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue(ImplementationLevel.getStereotype().equals("atpStructureElement"));
		assertTrue(ImplementationLevel.getStereotypeEx().equals("atpStructureElement"));
	}
	
	@Test
	public void testComment() {
		EAElement Comment = Util.findEAElementByName("Comment", eaModel);
		assertNotNull("Could not find Comment", Comment);
		assertTrue(Comment.getMetaType().equals("Class"));
		assertTrue(Comment.getStatus().equals("Proposed"));
		assertEquals(Comment.getPackage().getName(), "Elements");
		Comment.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Comment.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Comment must have 0 supertypes", 0, numberOfSuperTypes);
		assertTrue("Comment only has 1 Attribute", Comment.getAttributes().size() == 1);
		EAAttribute Body = Util.getEAAttributeForEAElement("body", Comment, eaModel);
		assertNotNull(Body);
		assertEquals(Body.getType(), "String");
	}
	
	@Test
	public void testEAElement() {
		EAElement EAElement = Util.findEAElementByName("EAElement", eaModel);
		assertNotNull("Could not find EAElement", EAElement);
		assertTrue(EAElement.getMetaType().equals("Class"));
		assertTrue(EAElement.getStatus().equals("Proposed"));
		assertEquals(EAElement.getPackage().getName(), "Elements");
		EAElement.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EAElement.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EAElement must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue("EAElement only has 1 Attribute", EAElement.getAttributes().size() == 1);
		EAAttribute name = Util.getEAAttributeForEAElement("name", EAElement, eaModel);
		assertNotNull(name);
		assertEquals(name.getType(), "String");
		
	}
	
	@Test
	public void testEAPackage() {
		EAElement EAPackage = Util.findEAElementByName("EAPackage", eaModel);
		assertNotNull("Could not find EAPackage", EAPackage);
		assertTrue(EAPackage.getMetaType().equals("Class"));
		assertTrue(EAPackage.getStatus().equals("Proposed"));
		assertEquals(EAPackage.getPackage().getName(), "Elements");
		EAPackage.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EAPackage.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EAPackage must have 1 supertypes", 1, numberOfSuperTypes);
	}
	
	@Test
	public void testArrayDatatype() {
		EAElement ArrayDatatype = Util.findEAElementByName("ArrayDatatype", eaModel);
		assertNotNull("Could not find ArrayDatatype", ArrayDatatype);
		assertTrue(ArrayDatatype.getMetaType().equals("Class"));
		assertTrue(ArrayDatatype.getStatus().equals("Proposed"));
		assertEquals(ArrayDatatype.getPackage().getName(), "Datatypes");
		ArrayDatatype.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : ArrayDatatype.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("ArrayDatatype must have 1 supertype", 1, numberOfSuperTypes);
		assertTrue("ArrayDatatype only has 2 Attributes", ArrayDatatype.getAttributes().size() == 2);
		EAAttribute maxLength = Util.getEAAttributeForEAElement("maxLength", ArrayDatatype, eaModel);
		assertNotNull(maxLength);
		assertEquals(maxLength.getType(), "Integer");
		EAAttribute minLength = Util.getEAAttributeForEAElement("minLength", ArrayDatatype, eaModel);
		assertNotNull(minLength);
		assertEquals(minLength.getType(), "Integer");
	}
	
	@Test
	public void testEANumerical() {
		EAElement EANumerical = Util.findEAElementByName("EANumerical", eaModel);
		assertNotNull("Could not find EANumerical", EANumerical);
		assertTrue(EANumerical.getMetaType().equals("Class"));
		assertTrue(EANumerical.getStatus().equals("Proposed"));
		assertEquals(EANumerical.getPackage().getName(), "Datatypes");
		EANumerical.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EANumerical.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EANumerical must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue("EANumerical only has 2 Attributes", EANumerical.getAttributes().size() == 2);
		EAAttribute max = Util.getEAAttributeForEAElement("max", EANumerical, eaModel);
		assertNotNull(max);
		assertEquals(max.getType(), "Numerical");
		EAAttribute min = Util.getEAAttributeForEAElement("min", EANumerical, eaModel);
		assertNotNull(min);
		assertEquals(min.getType(), "Numerical");
	}
	
	@Test
	public void testEnumeration() {
		EAElement Enumeration = Util.findEAElementByName("Enumeration", eaModel);
		assertNotNull("Could not find Enumeration", Enumeration);
		assertTrue(Enumeration.getMetaType().equals("Class"));
		assertTrue(Enumeration.getStatus().equals("Proposed"));
		assertEquals(Enumeration.getPackage().getName(), "Datatypes");
		Enumeration.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Enumeration.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Enumeration must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue("Enumeration only has 1 Attribute", Enumeration.getAttributes().size() == 1);
		EAAttribute isMultiValued = Util.getEAAttributeForEAElement("isMultiValued", Enumeration, eaModel);
		assertNotNull(isMultiValued);
		assertEquals(isMultiValued.getType(), "Boolean");
	}
	
	@Test
	public void testQuantity() {
		EAElement Quantity = Util.findEAElementByName("Quantity", eaModel);
		assertNotNull("Could not find Quantity", Quantity);
		assertTrue(Quantity.getMetaType().equals("Class"));
		assertTrue(Quantity.getStatus().equals("Proposed"));
		assertEquals(Quantity.getPackage().getName(), "Datatypes");
		Quantity.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Quantity.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Quantity must have 1 supertypes", 1, numberOfSuperTypes);
		assertTrue("Quantity only has 7 Attributes", Quantity.getAttributes().size() == 7);
		EAAttribute amountOfSubstanceExp = Util.getEAAttributeForEAElement("amountOfSubstanceExp", Quantity, eaModel);
		assertNotNull(amountOfSubstanceExp);
		assertEquals(amountOfSubstanceExp.getType(), "Integer");
		
		EAAttribute electricCurrentExp = Util.getEAAttributeForEAElement("electricCurrentExp", Quantity, eaModel);
		assertNotNull(electricCurrentExp);
		assertEquals(electricCurrentExp.getType(), "Integer");
		
		EAAttribute lengthExp = Util.getEAAttributeForEAElement("lengthExp", Quantity, eaModel);
		assertNotNull(lengthExp);
		assertEquals(lengthExp.getType(), "Integer");
		
		EAAttribute luminousIntensityExp = Util.getEAAttributeForEAElement("luminousIntensityExp", Quantity, eaModel);
		assertNotNull(luminousIntensityExp);
		assertEquals(luminousIntensityExp.getType(), "Integer");
		
		EAAttribute massExp = Util.getEAAttributeForEAElement("massExp", Quantity, eaModel);
		assertNotNull(massExp);
		assertEquals(massExp.getType(), "Integer");
		
		EAAttribute thermodynamicTemperatureExp = Util.getEAAttributeForEAElement("thermodynamicTemperatureExp", Quantity, eaModel);
		assertNotNull(thermodynamicTemperatureExp);
		assertEquals(thermodynamicTemperatureExp.getType(), "Integer");
		
		EAAttribute timeExp = Util.getEAAttributeForEAElement("timeExp", Quantity, eaModel);
		assertNotNull(timeExp);
		assertEquals(timeExp.getType(), "Integer");
	}
	
	@Test
	public void testRangeableValueType() {
		EAElement RangeableValueType = Util.findEAElementByName("RangeableValueType", eaModel);
		assertNotNull("Could not find RangeableValueType", RangeableValueType);
		assertTrue(RangeableValueType.getMetaType().equals("Class"));
		assertTrue(RangeableValueType.getStatus().equals("Proposed"));
		assertEquals(RangeableValueType.getPackage().getName(), "Datatypes");
		RangeableValueType.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : RangeableValueType.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("RangeableValueType must have 2 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("RangeableValueType only has 3 Attributes", RangeableValueType.getAttributes().size() == 3);
		EAAttribute accuracy = Util.getEAAttributeForEAElement("accuracy", RangeableValueType, eaModel);
		assertNotNull(accuracy);
		assertEquals(accuracy.getType(), "Float");
		
		EAAttribute resolution = Util.getEAAttributeForEAElement("resolution", RangeableValueType, eaModel);
		assertNotNull(resolution);
		assertEquals(resolution.getType(), "Float");
		
		EAAttribute significantDigits = Util.getEAAttributeForEAElement("significantDigits", RangeableValueType, eaModel);
		assertNotNull(significantDigits);
		assertEquals(significantDigits.getType(), "Integer");
	}
	
	@Test
	public void testUnit() {
		EAElement Unit = Util.findEAElementByName("Unit", eaModel);
		assertNotNull("Could not find Unit", Unit);
		assertTrue(Unit.getMetaType().equals("Class"));
		assertTrue(Unit.getStatus().equals("Proposed"));
		assertEquals(Unit.getPackage().getName(), "Datatypes");
		Unit.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Unit.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Unit must have 2 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("Unit only has 3 Attributes", Unit.getAttributes().size() == 3);
		EAAttribute factor = Util.getEAAttributeForEAElement("factor", Unit, eaModel);
		assertNotNull(factor);
		assertEquals(factor.getType(), "Float");
		
		EAAttribute symbol = Util.getEAAttributeForEAElement("symbol", Unit, eaModel);
		assertNotNull(symbol);
		assertEquals(symbol.getType(), "String");
		
		EAAttribute offset = Util.getEAAttributeForEAElement("offset", Unit, eaModel);
		assertNotNull(offset);
		assertEquals(offset.getType(), "Float");
	}
	
	@Test
	public void testEABooleanValue() {
		EAElement EABooleanValue = Util.findEAElementByName("EABooleanValue", eaModel);
		assertNotNull("Could not find EABooleanValue", EABooleanValue);
		assertTrue(EABooleanValue.getMetaType().equals("Class"));
		assertTrue(EABooleanValue.getStatus().equals("Proposed"));
		assertEquals(EABooleanValue.getPackage().getName(), "Values");
		EABooleanValue.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EABooleanValue.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EABooleanValue must have 1 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("EABooleanValue only has 1 Attribute", EABooleanValue.getAttributes().size() == 1);
		EAAttribute value = Util.getEAAttributeForEAElement("value", EABooleanValue, eaModel);
		assertNotNull(value);
		assertEquals(value.getType(), "Boolean");
	}
	
	@Test
	public void testEANumericalValue() {
		EAElement EANumericalValue = Util.findEAElementByName("EANumericalValue", eaModel);
		assertNotNull("Could not find EANumericalValue", EANumericalValue);
		assertTrue(EANumericalValue.getMetaType().equals("Class"));
		assertTrue(EANumericalValue.getStatus().equals("Proposed"));
		assertEquals(EANumericalValue.getPackage().getName(), "Values");
		EANumericalValue.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EANumericalValue.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EANumericalValue must have 1 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("EANumericalValue only has 1 Attribute", EANumericalValue.getAttributes().size() == 1);
		EAAttribute value = Util.getEAAttributeForEAElement("value", EANumericalValue, eaModel);
		assertNotNull(value);
		assertEquals(value.getType(), "Numerical");
	}
	
	@Test
	public void testEAStringValue() {
		EAElement EAStringValue = Util.findEAElementByName("EAStringValue", eaModel);
		assertNotNull("Could not find EAStringValue", EAStringValue);
		assertTrue(EAStringValue.getMetaType().equals("Class"));
		assertTrue(EAStringValue.getStatus().equals("Proposed"));
		assertEquals(EAStringValue.getPackage().getName(), "Values");
		EAStringValue.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : EAStringValue.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("EAStringValue must have 1 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("EAStringValue only has 1 Attribute", EAStringValue.getAttributes().size() == 1);
		EAAttribute value = Util.getEAAttributeForEAElement("value", EAStringValue, eaModel);
		assertNotNull(value);
		assertEquals(value.getType(), "String");
	}
	
	@Test
	public void testUserElementType() {
		EAElement UserElementType = Util.findEAElementByName("UserElementType", eaModel);
		assertNotNull("Could not find UserElementType", UserElementType);
		assertTrue(UserElementType.getMetaType().equals("Class"));
		assertTrue(UserElementType.getStatus().equals("Proposed"));
		assertEquals(UserElementType.getPackage().getName(), "UserAttributes");
		UserElementType.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : UserElementType.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("UserElementType must have 1 supertypes", 1, numberOfSuperTypes);
		
		assertTrue("UserElementType only has 2 Attributes", UserElementType.getAttributes().size() == 2);
		EAAttribute validFor = Util.getEAAttributeForEAElement("validFor", UserElementType, eaModel);
		assertNotNull(validFor);
		assertEquals(validFor.getType(), "String");
	
		EAAttribute key = Util.getEAAttributeForEAElement("key", UserElementType, eaModel);
		assertNotNull(key);
		assertEquals(key.getType(), "String");
	}
	
	@Test
	public void testReferrable() {
		EAElement Referrable = Util.findEAElementByName("Referrable", eaModel);
		assertNotNull("Could not find Referrable", Referrable);
		assertTrue(Referrable.getMetaType().equals("Class"));
		assertTrue(Referrable.getStatus().equals("Proposed"));
		assertEquals(Referrable.getPackage().getName(), "Identifiable");
		Referrable.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Referrable.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Referrable must have 0 supertype", 0, numberOfSuperTypes);
		
		assertTrue("Referrable only has 1 Attribute", Referrable.getAttributes().size() == 1);
		EAAttribute shortName = Util.getEAAttributeForEAElement("shortName", Referrable, eaModel);
		assertNotNull(shortName);
		assertEquals(shortName.getType(), "Identifier");
	}
		
		
	@Test
	public void testIdentifiable() {
		EAElement Identifiable = Util.findEAElementByName("Identifiable", eaModel);
		assertNotNull("Could not find Identifiable", Identifiable);
		assertTrue(Identifiable.getMetaType().equals("Class"));
		assertTrue(Identifiable.getStatus().equals("Proposed"));
		assertEquals(Identifiable.getPackage().getName(), "Identifiable");
		Identifiable.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : Identifiable.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("Identifiable must have 1 supertype", 1, numberOfSuperTypes);
		
		assertTrue("Identifiable only has 2 Attribute", Identifiable.getAttributes().size() == 2);
		EAAttribute category = Util.getEAAttributeForEAElement("category", Identifiable, eaModel);
		assertNotNull(category);
		assertEquals(category.getType(), "Identifier");
		
		EAAttribute uuid = Util.getEAAttributeForEAElement("uuid", Identifiable, eaModel);
		assertNotNull(uuid);
		assertEquals(uuid.getType(), "String");
	}

	@Test
	public void testSystem() {
		EAElement System = Util.findEAElementByName("System", eaModel);
		assertNotNull("Could not find System", System);
		assertTrue(System.getMetaType().equals("Class"));
		assertTrue(System.getStatus().equals("Proposed"));
		assertEquals(System.getPackage().getName(), "SystemTemplate");
		System.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : System.getConnectors()) {
			if (connector.getType().equals("Generalization"))
				numberOfSuperTypes++;
		}
		assertEquals("System must have 2 supertype", 2, numberOfSuperTypes);
				
		assertTrue("System only has 2 Attributess", System.getAttributes().size() == 2);
		EAAttribute ecuExtractVersion = Util.getEAAttributeForEAElement("ecuExtractVersion", System, eaModel);
		assertNotNull(ecuExtractVersion);
		assertEquals(ecuExtractVersion.getType(), "RevisionLabelString");
					
		EAAttribute systemVersion = Util.getEAAttributeForEAElement("systemVersion", System, eaModel);
		assertNotNull(systemVersion);
		assertEquals(systemVersion.getType(), "RevisionLabelString");
	}
				

	@Test
	public void testFunctionClientServerPort() {
		EAElement FunctionClientServerPort = Util.findEAElementByName("FunctionClientServerPort", eaModel);
		assertNotNull("Could not find FunctionClientServerPort", FunctionClientServerPort);
		assertTrue(FunctionClientServerPort.getMetaType().equals("Class"));
		assertTrue(FunctionClientServerPort.getStatus().equals("Proposed"));
		assertEquals(FunctionClientServerPort.getPackage().getName(), "FunctionModeling");
		FunctionClientServerPort.getConnectors();
		int numberOfSuperTypes = 0;
		for (EAConnector connector : FunctionClientServerPort.getConnectors()) {
			if (connector.getType().equals("Generalization"))
							numberOfSuperTypes++;
		}
		assertEquals("FunctionClientServerPort must have 1 supertype", 1, numberOfSuperTypes);
		assertTrue("FunctionClientServerPort only has 1 Attributess", FunctionClientServerPort.getAttributes().size() == 1);
		EAAttribute kind = Util.getEAAttributeForEAElement("kind", FunctionClientServerPort, eaModel);
		assertNotNull(kind);
		assertEquals(kind.getType(), "ClientServerKind");
	}
}
