/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EStructuralFeature.Setting;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil.UsageCrossReferencer;

import eaadapter.common.util.ModelConstants;
import eaadapter.common.util.ModelUtil;
import eaadapter.xsd.common.ModelInformation;
import eaadapter.xsd.common.ModelInformationFactory;
import eaadapter.xsd.model.CustomClassType;
import eaadapter.xsd.model.CustomObjectFactory;
import eaadapter.xsd.model.EAXSDAbstract;
import eaadapter.xsd.model.EAXSDAdministrativeData;
import eaadapter.xsd.model.EAXSDAnnotation;
import eaadapter.xsd.model.EAXSDAttribute;
import eaadapter.xsd.model.EAXSDCardinality;
import eaadapter.xsd.model.EAXSDCase01;
import eaadapter.xsd.model.EAXSDCase02;
import eaadapter.xsd.model.EAXSDCase05;
import eaadapter.xsd.model.EAXSDCase06;
import eaadapter.xsd.model.EAXSDCase07;
import eaadapter.xsd.model.EAXSDCase08;
import eaadapter.xsd.model.EAXSDCase09;
import eaadapter.xsd.model.EAXSDCase10;
import eaadapter.xsd.model.EAXSDCase13;
import eaadapter.xsd.model.EAXSDCase15;
import eaadapter.xsd.model.EAXSDCase17;
import eaadapter.xsd.model.EAXSDCase18;
import eaadapter.xsd.model.EAXSDCase19;
import eaadapter.xsd.model.EAXSDCase20;
import eaadapter.xsd.model.EAXSDCase21;
import eaadapter.xsd.model.EAXSDCase22;
import eaadapter.xsd.model.EAXSDCase23;
import eaadapter.xsd.model.EAXSDCase24;
import eaadapter.xsd.model.EAXSDClassType;
import eaadapter.xsd.model.EAXSDConstraint;
import eaadapter.xsd.model.EAXSDDerived;
import eaadapter.xsd.model.EAXSDEnumeration;
import eaadapter.xsd.model.EAXSDEnumerationLiteral;
import eaadapter.xsd.model.EAXSDGlobal;
import eaadapter.xsd.model.EAXSDModel;
import eaadapter.xsd.model.EAXSDPair;
import eaadapter.xsd.model.EAXSDReference;
import eaadapter.xsd.model.EAXSDSimpleAttribute;
import eaadapter.xsd.model.EAXSDSimpleType;
import eaadapter.xsd.model.EAXSDSubtypes;
import eaadapter.xsd.model.EAXSDTag;
import eaadapter.xsd.model.EAXSDType;
import eaadapter.xsd.util.TaggedValueHelper;

public class Ecore2XSDModel {
    
    /**
     * Sorts a list of ENamedElements by type and by name.
     */
    private static Comparator<ENamedElement> E_NAMED_ELEMENT_TYPE_NAME_COMPARATOR = new Comparator<ENamedElement>() {

        private int getValue(ENamedElement n) {
            if (n instanceof EClass) {
                return 0;
            } else if (n instanceof EDataType) {
                return 1;
            } else {
                return 2;
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public int compare(ENamedElement e1, ENamedElement e2) {
            int v1 = getValue(e1);
            int v2 = getValue(e2);

            if (v1 == v2) {
                return e1.getName().compareTo(e2.getName());
            } else if (v1 > v2) {
                return 1;
            } else {
                return -1;
            }
        }
    };
    
    /**
     * Sorts a list of ENamedElements by their names.
     */
    private static Comparator<ENamedElement> E_NAMED_ELEMENT_NAME_COMPARATOR = 
            new Comparator<ENamedElement>() {

        /**
         * {@inheritDoc}
         */
        @Override
        public int compare(ENamedElement e1, ENamedElement e2) {
            return e1.getName().compareTo(e2.getName());
        }
    };
    
    private enum ClassType {
        NOT_PRIMITIVE, 
        NATIVE_PRIMITIVE, 
        CUSTOM_PRIMITIVE 
    }
    
    private enum XMLMappingKind {
        GENERATE_ATTRIBUTE, //structural feature is mapped as XML attribute
        GENERATE_ELEMENT //structural feature is mapped as XML element
    }
    
    private final Map <String, CustomClassType> classTypes;
    private final CustomObjectFactory factory;
    private final List <EAXSDType> types;

    public Ecore2XSDModel() {
        types = new ArrayList <EAXSDType>();
        factory = new CustomObjectFactory();
        classTypes = new Hashtable <String, CustomClassType>();
    }
    
    /**
     * Fills a list with EObject instance while traversing the hierarchy of the 
     * given package
     * 
     * @param list The list used to buffer the instances. 
     * @param ePackage The EPackage which has to be traversed.
     */
    private void addENamedElements(List<ENamedElement> list, EPackage ePackage) {
        for (Iterator<EObject> iter = ePackage.eAllContents(); iter.hasNext();) {
            EObject obj = iter.next();
            if (obj instanceof EClass || obj instanceof EEnum
                    || obj instanceof EDataType) {
                list.add((ENamedElement) obj);
            }
        }
    }
    
    /**
     * Returns a list with all ENamedElement instances of the given package
     * 
     * @param pck The EPackage which shall be traversed.
     * @return A list of ENamedElement
     */
    private List<ENamedElement> getENamedElements(EPackage pck) {
        List<ENamedElement> list = new ArrayList<ENamedElement>();
        addENamedElements(list, (EPackage) pck);
        return list;
    }
    
    /**
     * Converts all elements contained by the supplied model
     * 
     * @param schemaModel 
     *              The model which buffers the XSD description of the converted 
     *              elements
     * @param model
     *              The model which will be converted
     */
    private void convertModel(EAXSDModel schemaModel, EPackage model) {
        List<ENamedElement> objects = getENamedElements(model);
        Collections.sort(objects, E_NAMED_ELEMENT_TYPE_NAME_COMPARATOR);
        for (int i = 0; i < objects.size(); i++) {
            ENamedElement eobject = objects.get(i);
            if (eobject instanceof EClass) {
                convertClass(schemaModel, (EClass) eobject);
            } else if (eobject instanceof EEnum) {
                convertEnum(schemaModel, (EEnum) eobject);
            } else {
                convertDataType(schemaModel, (EDataType) eobject);
            }
        }
    }
    
    /**
     * Generates the schema description for a EClass
     * 
     * @param schemaModel 
     *          The model which buffers the XSD description of the converted 
     *          EClass
     * @param eClass EClass which will be converted.
     */
    private void convertClass (EAXSDModel schemaModel, EClass eClass) {
        if (ModelUtil.hasStereotype(eClass, 
                ModelConstants.STEREOTYPE_PRIMITIVE)) {
            if (eClass.getEAllAttributes().size() != 0) {
                convertSimpleTypeWithAttributes( schemaModel, eClass );
            } else {
                // do nothing
            }
        } else {
            CustomClassType clsType = (CustomClassType) 
                    factory.createEAXSDClassType();
            
            clsType.setType(createEAXSDType(eClass));
            clsType.setAnnotation(convertAnnotation(eClass));
            clsType.setGlobal(isGlobalElement(eClass));
            clsType.setAbstract(eClass.isAbstract());
            clsType.setReferenced(isReferenced(eClass));
            clsType.setOrdered(isOrdered(eClass));
            
            classTypes.put(clsType.getType().getXmlname(), clsType);

            List <EStructuralFeature> features = getDirectFeatures(eClass, 
                    XMLMappingKind.GENERATE_ELEMENT);
            
            convertFeaturesToElements( clsType, features );

            convertFeaturesToAttributes(clsType.getSimpleattribute(), eClass);

            if (!eClass.isAbstract()) {
                convertAbstract(clsType, eClass);
            }

            if (clsType.isReferenced()) {
                convertSubtypes(clsType, eClass);
            }

            if (clsType.isGlobal()) {
                convertGlobal(clsType, eClass);
            }

            schemaModel.getTypes().add(clsType);

        }
    }
    
    /**
     * Converts all suptypes of the given eClass and adds them to the subtypes 
     * list of the given classType
     * 
     * @param classType which has a list of subtypes
     * @param eClass which is the base class of all subtypes
     */
    private void convertSubtypes(EAXSDClassType classType, EClass eClass) {
        EAXSDSubtypes subTypes = factory.createEAXSDSubtypes();
        subTypes.setType(createEAXSDType(eClass));

        Set<EClass> concreteTypes = new HashSet<EClass>(ModelUtil
                .getConcreteSubclasses(eClass));
        
        List<EClass> allTypes = ModelUtil.getSubclasses(eClass);
        for (EClass type : allTypes) {
            EAXSDType arxsdtype = createEAXSDType(type);
            arxsdtype.setConcrete(Boolean.valueOf(concreteTypes.contains(type)));
            subTypes.getSubtype().add(arxsdtype);
            concreteTypes.remove(type);
        }

        classType.setSubtypes(subTypes);
    }
    
    /**
     * Converts the given eClass to a global type
     * 
     * @param classType which consists of a global type
     * @param eClass which will be converted to global type
     */
    private void convertGlobal(EAXSDClassType classType, EClass eClass) {
        EAXSDGlobal global = factory.createEAXSDGlobal();
        global.setPackage(eClass.getEPackage().getName());
        global.setType(createEAXSDType(eClass));
        global.setAnnotation(convertAnnotation(eClass));
        
        classType.setGlobaltype(global);
    }
    
    /**
     * Converts the given abstract eClass to an abstract type
     * 
     * @param classType which consists of an abstract type
     * @param eClass which shall be converted
     */
    private void convertAbstract(EAXSDClassType classType, EClass eClass) {
        EAXSDAbstract abstractType = factory.createEAXSDAbstract();
        abstractType.setAnnotation(convertAnnotation(eClass));
        abstractType.setTypename(getXmlElementName(eClass));
        abstractType.setModelname(eClass.getName());
        abstractType.setPackage(eClass.getEPackage().getName());
        abstractType.setAttributes(getDirectFeatures(eClass, 
                XMLMappingKind.GENERATE_ATTRIBUTE).size() > 0);
        
        EStructuralFeature feature = getSimpleContentFeature(eClass);
        abstractType.setSimple(feature != null);

        List<EClass> baseClasses = ModelUtil.getBaseClasses(eClass);
        
        for (EClass baseClass : baseClasses) {
            if (getDirectFeatures(baseClass, 
                    XMLMappingKind.GENERATE_ATTRIBUTE).size() > 0) {
                abstractType.getBaseclasswithattributes().add(
                        createEAXSDType(baseClass));
            }
        }
        
        if (feature != null) {
            abstractType.setType(createEAXSDType(eClass));
            abstractType.setBasetype(createEAXSDType(feature));
        } else {
            abstractType.setType(createEAXSDType(eClass));
            abstractType.setMixed(ModelUtil.hasStereotype(eClass, 
                    ModelConstants.STEREOTYPE_ATPMIXEDSTRING));
            abstractType.setOrdered(isOrdered(eClass));
            
            for (EClass baseClass : baseClasses) {
                abstractType.getBaseclass().add(createEAXSDType(baseClass));
            }
        }
        
        classType.setAbstracttype(abstractType);
    }
    
    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * 
     * There are several cases for this import which depend on the combination 
     * of the following attributes of the given reference:
     * <li>hasRoleElement<li>
     * <li>hasRoleWrapperElement<li>
     * <li>hasTypeElement<li>
     * <li>hasTypeWrapperElement<li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertComposite(EAXSDClassType classType, EReference reference) {
        if (reference.getEReferenceType() != null) {

            boolean roleElement = hasRoleElement(reference);
            boolean typeElement = hasTypeElement(reference);
            boolean typeWrapperElement = hasTypeWrapperElement(reference);
            boolean roleWrapperElement = hasRoleWrapperElement(reference);

            if (roleWrapperElement && roleElement && typeWrapperElement
                    && typeElement) {
                convertCase01(classType, reference);
            } else if (roleWrapperElement && roleElement && typeWrapperElement
                    && !typeElement) {
                convertCase02(classType, reference);
            } else if (roleWrapperElement && roleElement && !typeWrapperElement
                    && typeElement) {
                convertCase03(classType, reference);
            } else if (roleWrapperElement && roleElement && !typeWrapperElement
                    && !typeElement) {
                convertCase04(classType, reference);
            } else if (roleWrapperElement && !roleElement && typeWrapperElement
                    && typeElement) {
                convertCase05(classType, reference);
            } else if (roleWrapperElement && !roleElement && typeWrapperElement
                    && !typeElement) {
                convertCase06(classType, reference);
            } else if (roleWrapperElement && !roleElement
                    && !typeWrapperElement && typeElement) {
                convertCase07(classType, reference);
            } else if (roleWrapperElement && !roleElement
                    && !typeWrapperElement && !typeElement) {
                convertCase08(classType, reference);
            } else if (!roleWrapperElement && roleElement && typeWrapperElement
                    && typeElement) {
                convertCase09(classType, reference);
            } else if (!roleWrapperElement && roleElement && typeWrapperElement
                    && !typeElement) {
                convertCase10(classType, reference);
            } else if (!roleWrapperElement && roleElement
                    && !typeWrapperElement && typeElement) {
                convertCase11(classType, reference);
            } else if (!roleWrapperElement && roleElement
                    && !typeWrapperElement && !typeElement) {
                convertCase12(classType, reference);
            } else if (!roleWrapperElement && !roleElement
                    && typeWrapperElement && typeElement) {
                convertCase13(classType, reference, typeElement);
            } else if (!roleWrapperElement && !roleElement
                    && typeWrapperElement && !typeElement) {
                convertCase13(classType, reference, typeElement);
            } else if (!roleWrapperElement && !roleElement
                    && !typeWrapperElement && typeElement) {
                convertCase15(classType, reference, typeElement);
            } else if (!roleWrapperElement && !roleElement
                    && !typeWrapperElement && !typeElement) {
                convertCase15(classType, reference, typeElement);
            }
        }
    }
    
    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case01:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase01(EAXSDClassType classType, 
            EReference reference) {
        
        EAXSDCase01 case01 = factory.createEAXSDCase01();
        
        case01.setElementname(getXmlElementName(reference));
        case01.setRolename(getXmlElementPluralName(reference));
        case01.setCardinality(convertCardinality(reference));
        case01.setAnnotation(convertAnnotation(reference));
        
        List<EClass> subClasses = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : subClasses) {
            case01.getElement().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case01);
    }

    
    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case02:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase02(EAXSDClassType classType, 
            EReference reference) {
        
        EAXSDCase02 case02 = factory.createEAXSDCase02();
        
        case02.setElementname(getXmlElementName(reference));
        case02.setRolename(getXmlElementPluralName(reference));
        case02.setCardinality(convertCardinality(reference));
        case02.setAnnotation(convertAnnotation(reference));
        
        List<EClass> subClasses = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : subClasses) {
            case02.getElement().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case02);
    }
    
    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case19:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * <li>types.length > 1</li>
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted 
     * @param types a list of concrete subclasses of the reference type
     */
    private void convertCase19(EAXSDClassType classType, EReference reference,
            List<EClass> types) {
        
        EAXSDCase19 case19 = factory.createEAXSDCase19();
        
        case19.setElementname(getXmlElementName(reference));
        case19.setRolename(getXmlElementPluralName(reference));
        case19.setAnnotation(convertAnnotation(reference));
        case19.setCardinality(convertCardinality(reference));

        for (EClass type : types) {
            case19.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case19);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case20:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * <li>types.length = 1</li>
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param type the only concrete subclass of the reference type
     */
    private void convertCase20(EAXSDClassType classType, EReference reference,
            EClass type) {
        
        EAXSDCase20 case20 = factory.createEAXSDCase20();
        
        case20.setElementname(getXmlElementName(reference));
        case20.setRolename(getXmlElementPluralName(reference));
        case20.setAnnotation(convertAnnotation(reference));
        case20.setCardinality(convertCardinality(reference));
        case20.setType(createEAXSDType(type));
        
        classType.getElementattributes().add(case20);
    }

    // roleWrapperElement && roleElement && ! typeWrapperElement && typeElement
    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case03:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * This case will be split up into another two cases which depends on the size
     * of the list of the concrete subclasses.
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase03(EAXSDClassType classType, EReference reference) {
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        if (types.size() > 1) {
            convertCase19(classType, reference, types);
        } else {
            convertCase20(classType, reference, types.get(0));
        }
    }
    
    /**
     * Convert the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case17:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * <li>types.length > 1</li>
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param types a list of concrete subclasses of the reference type
     */
    private void convertCase17(EAXSDClassType classType, EReference reference,
            List<EClass> types) {

        EAXSDCase17 case17 = factory.createEAXSDCase17();
        
        case17.setElementname(getXmlElementName(reference));
        case17.setRolename(getXmlElementPluralName(reference));
        case17.setAnnotation(convertAnnotation(reference));
        case17.setCardinality(convertCardinality(reference));
        
        
        for (EClass type : types) {
            case17.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case17);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case18:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li>
     * <li>types.length = 1 </li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param type the only concrete subclass of the reference type
     */
    private void convertCase18(EAXSDClassType classType, EReference reference,
            EClass type) {

        EAXSDCase18 case18 = factory.createEAXSDCase18();
        
        case18.setElementname(getXmlElementName(reference));
        case18.setRolename(getXmlElementPluralName(reference));
        case18.setAnnotation(convertAnnotation(reference));
        case18.setCardinality(convertCardinality(reference));
        
        case18.setType(createEAXSDType(type));
        
        classType.getElementattributes().add(case18);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case04:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * This case will be split up into another two cases which depends on the size
     * of the list of the concrete subclasses.
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase04(EAXSDClassType classType, EReference reference) {
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(
                reference.getEReferenceType());
        
        if (types.size() > 1) {
            convertCase17(classType, reference, types);
        } else {
            convertCase18(classType, reference, types.get(0));
        }
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case05:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase05(EAXSDClassType classType, EReference refrence) {
        
        EAXSDCase05 case05 = factory.createEAXSDCase05();
        case05.setRolename(getXmlElementPluralName(refrence));
        case05.setAnnotation(convertAnnotation(refrence));
        case05.setCardinality(convertCardinality(refrence));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(refrence
                .getEReferenceType());
        
        for (EClass type : types) {
            case05.getReferred().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case05);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case06:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase06(EAXSDClassType classType, EReference reference) {

        EAXSDCase06 case06 = factory.createEAXSDCase06();
        case06.setRolename(getXmlElementPluralName(reference));
        case06.setAnnotation(convertAnnotation(reference));
        case06.setCardinality(convertCardinality(reference));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            case06.getReferred().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case06);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case07:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase07(EAXSDClassType classType, EReference refrence) {

        EAXSDCase07 case07 = factory.createEAXSDCase07();
        case07.setRolename(getXmlElementPluralName(refrence));
        case07.setAnnotation(convertAnnotation(refrence));
        case07.setCardinality(convertCardinality(refrence));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(refrence
                .getEReferenceType());
        
        for (EClass type : types) {
            case07.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case07);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case08:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>true</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase08(EAXSDClassType classType, EReference reference) {
        
        EAXSDCase08 case08 = factory.createEAXSDCase08();
        case08.setRolename(getXmlElementPluralName(reference));
        case08.setAnnotation(convertAnnotation(reference));
        case08.setCardinality(convertCardinality(reference));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            case08.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case08);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case09:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase09(EAXSDClassType classType, EReference reference) {
        
        EAXSDCase09 case09 = factory.createEAXSDCase09();
        case09.setAnnotation(convertAnnotation(reference));
        case09.setRolename(getXmlElementName(reference));
        case09.setCardinality(convertCardinality(reference));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            case09.getReferred().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case09);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case10:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase10(EAXSDClassType classType, EReference reference) {
        
        EAXSDCase10 case10 = factory.createEAXSDCase10();
        case10.setRolename(getXmlElementName(reference));
        case10.setAnnotation(convertAnnotation(reference));
        case10.setCardinality(convertCardinality(reference));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            case10.getReferred().add(createEAXSDPair(type));
        }
        
        classType.getElementattributes().add(case10);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case21:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * <li>types.lenght > 1</li>
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param types a list of concrete subclasses of the reference type
     */
    private void convertCase21(EAXSDClassType classType, EReference reference,
            List<EClass> types) {

        EAXSDCase21 case21 = factory.createEAXSDCase21();
        case21.setElementname(getXmlElementName(reference));
        case21.setAnnotation(convertAnnotation(reference));
        case21.setCardinality(convertCardinality(reference));
        
        
        for (EClass type : types) {
            case21.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case21);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case22:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li>
     * <li>types.length = 1</li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param type the only concrete subclass of the reference type
     */
    private void convertCase22(EAXSDClassType classType, EReference reference,
            EClass type) {

        EAXSDCase22 case22 = factory.createEAXSDCase22();
        case22.setElementname(getXmlElementName(reference));
        case22.setAnnotation(convertAnnotation(reference));
        case22.setCardinality(convertCardinality(reference));
        
        case22.setType(createEAXSDType(type));
        
        classType.getElementattributes().add(case22);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case11:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * This case will be split up into another two cases which depends on the size
     * of the list of the concrete subclasses.
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase11(EAXSDClassType classType, EReference reference) {
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        if (types.size() > 1) {
            convertCase21(classType, reference, types);
        } else {
            convertCase22(classType, reference, types.get(0));
        }
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case23:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * <li>types.length > 1</li>
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param types a list of concrete subclasses of the reference type
     */
    private void convertCase23(EAXSDClassType classType, EReference reference,
            List<EClass> types) {
        
        EAXSDCase23 case23 = factory.createEAXSDCase23();
        case23.setRolename(getXmlElementName(reference));
        case23.setAnnotation(convertAnnotation(reference));
        case23.setCardinality(convertCardinality(reference));
        
        for (EClass type : types) {
            case23.getReferred().add(getXmlElementName(type));
        }
        
        classType.getElementattributes().add(case23);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case24:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param type the only concrete subclass of the reference type
     */
    private void convertCase24(EAXSDClassType classType, EReference reference,
            EClass type) {
        
        EAXSDCase24 case24 = factory.createEAXSDCase24();
        case24.setRolename(getXmlElementName(reference));
        case24.setAnnotation(convertAnnotation(reference));
        case24.setCardinality(convertCardinality(reference));
        
        case24.setType(createEAXSDType(type));
        classType.getElementattributes().add(case24);
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case12:
     * 
     * <li>hasRoleElement -> <code>true</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * This case will be split up into another two cases which depends on the size
     * of the list of the concrete subclasses.
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     */
    private void convertCase12(EAXSDClassType classType, EReference reference) {
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());

        if (types.size() > 1) {
            convertCase23(classType, reference, types);
        } else {
            convertCase24(classType, reference, types.get(0));
        }
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case13:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code> or <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>true</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param typeElement indicates if the reference has a type element
     */
    private void convertCase13(EAXSDClassType classType, EReference reference,
            boolean typeElement) {
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            EAXSDCase13 case13 = factory.createEAXSDCase13();
            case13.setReferred(getXmlElementName(reference));
            case13.setType(createEAXSDType(type));
            case13.setCardinality(convertCardinality(reference));
            case13.setAnnotation(convertAnnotation(reference));
            case13.setGroupref(!typeElement);
            
            classType.getElementattributes().add(case13);
        }
    }

    /**
     * Converts the given reference to an (element) attribute and adds it to 
     * the attributes list of the given classType.
     * Case15:
     * 
     * <li>hasRoleElement -> <code>false</code></li>
     * <li>hasRoleWrapperElement -> <code>false</code></li>
     * <li>hasTypeElement -> <code>true</code> or <code>false</code></li>
     * <li>hasTypeWrapperElement -> <code>false</code></li> 
     * 
     * @param classType which has a list of (element) attributes
     * @param reference which will be converted
     * @param typeElement indicates if the reference has a type element
     */
    private void convertCase15(EAXSDClassType classType, EReference reference,
            boolean typeElement) {
        
        EAXSDCase15 case15 = factory.createEAXSDCase15();
        case15.setCardinality(convertCardinality(reference));
        case15.setAnnotation(convertAnnotation(reference));
        
        List<EClass> types = ModelUtil.getConcreteSubclasses(reference
                .getEReferenceType());
        
        for (EClass type : types) {
            if (typeElement || getDirectFeatures(
                    type, XMLMappingKind.GENERATE_ELEMENT).size() > 0) {
                case15.getReferred().add(getXmlElementName(type));
            }
        }
        
        case15.setGroupref(!typeElement);
        classType.getElementattributes().add(case15);
    }
    
    /**
     * Creates a new EAXSDPair for the given eClass.
     * 
     * @param eClass
     * @return new EAXSDPair
     */
    private EAXSDPair createEAXSDPair(EClass eClass) {
        EAXSDPair res = factory.createEAXSDPair();
        
        res.setSecond(getXmlElementName(eClass));
        res.setFirst(getXmlElementPluralName(eClass));
        
        return res;
    }
    
    /**
     * Converts the given list of structural features to an (element) attributes
     * and adds them to the attributes list of the given classType.
     * 
     * @param classType which has a list of (element) attributes
     * @param features List of structural features.
     */
    private void convertFeaturesToElements(EAXSDClassType classType,
            List<EStructuralFeature> features) {
        
        // sort features according by their name
        EStructuralFeature[] structuralFeatures = 
                new EStructuralFeature[features.size()];
        features.toArray(structuralFeatures);
        Arrays.sort(structuralFeatures, E_NAMED_ELEMENT_NAME_COMPARATOR);
        
        // import features
        for (int i = 0; i < structuralFeatures.length; i++) {
            if (structuralFeatures[i] instanceof EAttribute
                    || (structuralFeatures[i] instanceof EReference
                        && ModelUtil.hasStereotype(
                                structuralFeatures[i].getEType(), 
                                ModelConstants.STEREOTYPE_PRIMITIVE))) {
                convertAttribute(classType, structuralFeatures[i]);
            } else {
                EReference ref = (EReference) structuralFeatures[i];
                
                if (isDerived(ref)) {
                    convertDerived(classType, ref);
                } else {
                    if (ref.isContainment()) {
                        convertComposite(classType, ref);
                    } else {
                        convertReference(classType, ref);
                    }
                }
            }
        }
    }
    
    /**
     * Converts the given reference and adds it to the attributes list of
     * the given classType
     * 
     * @param classType which has a list of attributes
     * @param reference which will be converted
     */
    private void convertReference(EAXSDClassType classType, EReference reference) {
        
        if (reference.getEType() != null) {
            EAXSDReference eaxsdRef = factory.createEAXSDReference();
            
            eaxsdRef.setCardinality(convertCardinality(reference));
            eaxsdRef.setAnnotation(convertAnnotation(reference));
            eaxsdRef.setWrapper(hasRoleWrapperElement(reference));
            eaxsdRef.setDerived(isDerived(reference));
            
            if (eaxsdRef.isDerived()) {
                eaxsdRef.setName(reference.getName());
                eaxsdRef.setRolename(getXmlElementName(reference));
            } else if (eaxsdRef.isWrapper()) {
                eaxsdRef.setElementname(getXmlElementName(reference));
                eaxsdRef.setRolename(getXmlElementPluralName(reference));
                eaxsdRef.setType(createEAXSDType(reference));
            } else {
                eaxsdRef.setElementname(getXmlElementName(reference));
                eaxsdRef.setRolename(getXmlElementName(reference));
                eaxsdRef.setType(createEAXSDType(reference));
            }
            
            classType.getElementattributes().add(eaxsdRef);
        }
    }
    
    /**
     * Converts the given derived reference and adds it to the attributes list of
     * the given classType
     * 
     * @param classType which has a list of attributes
     * @param reference which will be converted
     */
    private void convertDerived(EAXSDClassType classType, EReference reference) {
        EAXSDDerived derived = factory.createEAXSDDerived();
        derived.setName(reference.getName());
        classType.getElementattributes().add(derived);
    }
    
    /**
     * Converts the given attribute and adds it to the attributes list of the 
     * given classType.
     * 
     * @param classType which has a list of attributes
     * @param attribute which will be converted
     */
    private void convertAttribute(EAXSDClassType classType,
            EStructuralFeature attribute) {
        EAXSDAttribute attr = factory.createEAXSDAttribute();
        attr.setAnnotation(convertAnnotation(attribute));

        EAXSDCardinality cardinality = convertCardinality(attribute);
        attr.setRolename(getXmlElementPluralName(attribute));
        attr.setElementname(getXmlElementName(attribute));
        attr.setCardinality(cardinality);
        attr.setType(createEAXSDType(attribute));
        attr.setWrapper(hasRoleWrapperElement(attribute));
        
        classType.getElementattributes().add(attr);
    }
    
    /**
     * Converts the given eClass to a simple type and adds it to the given schemaModel
     * 
     * @param schemaModel 
     *              The model which buffers the XSD description of the element
     * @param eClass which will be converted
     */
    private void convertSimpleTypeWithAttributes (EAXSDModel schemaModel, 
            EClass eClass) {
        TaggedValueHelper helper = new TaggedValueHelper(eClass);
        String customType = helper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_XSD_CUSTOM_TYPE);
        
        if (customType != null) {

            EAXSDSimpleType simpletype = factory.createEAXSDSimpleType();

            simpletype.setCustomtype(customType);
            simpletype.setAnnotation(convertAnnotation(eClass));

            String baseType =  helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_TYPE, "string");

            simpletype.setBasetype(baseType);

            simpletype.setMininclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_INCLUSIVE));
            simpletype.setMaxinclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_INCLUSIVE ));
            simpletype.setMinexclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_EXCLUSIVE));
            simpletype.setMaxexclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_EXCLUSIVE));
            simpletype.setMinlength(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_LENGTH));
            simpletype.setMaxlength(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_LENGTH));
            simpletype.setTotaldigits( helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_TOTAL_DIGITS));
            simpletype.setFractiondigits(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_FRACTION_DIGITS));
            simpletype.setLen(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_LENGTH));
            simpletype.setWhitespace(helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_WHITESPACE));
            simpletype.setPattern(helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_PATTERN));

            simpletype.setRestrictions((simpletype.getMininclusive() != null)
                    || (simpletype.getMaxinclusive() != null)
                    || (simpletype.getMinexclusive() != null)
                    || (simpletype.getMaxexclusive() != null)
                    || (simpletype.getMinlength() != null)
                    || (simpletype.getMaxlength() != null)
                    || (simpletype.getTotaldigits() != null)
                    || (simpletype.getFractiondigits() != null)
                    || (simpletype.getLen() != null)
                    || (simpletype.getWhitespace() != null)
                    || (simpletype.getPattern() != null));

            convertFeaturesToAttributes(simpletype.getSimpleattribute(), eClass);

            schemaModel.getTypes().add(simpletype);
        }
    }
    

    /**
     * Converts features of the given eClass to simple attributes and adds 
     * them to given list.
     * 
     * @param list which holds the generated EAXSDSimpleAttribute's
     * @param eClass contains the features which shall be converted to attributes
     */
    private void convertFeaturesToAttributes(List<EAXSDSimpleAttribute> list,
            EClass eClass) {
        List<EStructuralFeature> features = getDirectFeatures(eClass,
                XMLMappingKind.GENERATE_ATTRIBUTE);
        if (!features.isEmpty()) {
            convertFeaturesToAttributes(list, features);
        }
    }
    
    /**
     * Returns all structural features of the given class, depending on their
     * XML mapping. 
     * 
     * @param eClass 
     *          Class whose features are returned.
     * @param xmlMappingKind 
     *          the XML mapping kind of the structural features in the returned 
     *          subset
     * @return subset of structural features
     */
    private List<EStructuralFeature> getFeatures(EClass eClass,
            XMLMappingKind xmlMappingKind) {
        List<EStructuralFeature> features = new LinkedList<EStructuralFeature>(
                eClass.getEAllStructuralFeatures());

        boolean isXmlElement = xmlMappingKind == XMLMappingKind.GENERATE_ELEMENT;

        Iterator<EStructuralFeature> featureIter = features.iterator();
        while (featureIter.hasNext()) {
            EStructuralFeature feature = featureIter.next();

            TaggedValueHelper tvHelper = new TaggedValueHelper(feature);

            boolean isXmlAttribute = tvHelper.getBooleanValue(
                    ModelConstants.TAGGED_VALUE_XML_ATTRIBUTE, Boolean.FALSE)
                    .booleanValue();

            if (isXmlElement == isXmlAttribute) {
                featureIter.remove();
            }
        }

        return features;
    }
    
    /**
     * Returns direct structural features of the given class, depending on their
     * XML mapping. 
     * 
     * @param eClass 
     *          Class whose features are returned.
     * @param xmlMappingKind 
     *          the XML mapping kind of the structural features in the returned 
     *          subset
     * @return subset of structural features
     */
    private List<EStructuralFeature> getDirectFeatures(EClass eClass, 
            XMLMappingKind xmlMappingKind) {
        
        List<EStructuralFeature> features = new LinkedList<EStructuralFeature>(
                eClass.getEStructuralFeatures());
        
        boolean isXmlElement = xmlMappingKind == XMLMappingKind.GENERATE_ELEMENT;
        
        Iterator<EStructuralFeature> featureIter = features.iterator();
        while (featureIter.hasNext()) {
            EStructuralFeature feature = featureIter.next();
            
            TaggedValueHelper tvHelper = new TaggedValueHelper(feature);

            boolean isXmlAttribute = tvHelper.getBooleanValue(
                    ModelConstants.TAGGED_VALUE_XML_ATTRIBUTE, 
                    Boolean.FALSE).booleanValue();
            
            if (isXmlElement == isXmlAttribute) {
                featureIter.remove();
            }
        }
        
        return features;
    }
    
    /**
     * Converts the given list of structural features to simple attributes and adds 
     * them to given list.
     * 
     * @param structuralFeatures list of structural features.
     * @param list which holds the generated EAXSDSimpleAttribute's
     */
    private void convertFeaturesToAttributes (List<EAXSDSimpleAttribute> list,
            List<EStructuralFeature> structuralFeatures) {
        
        // sort features by their name
        EStructuralFeature[] features = 
                new EStructuralFeature[structuralFeatures.size()];
        structuralFeatures.toArray(features);
        Arrays.sort( features, new Comparator<EStructuralFeature>() {
            @Override
            public int compare(EStructuralFeature o1, EStructuralFeature o2) {
                if (o1 != null && o2 != null) {
                    return o1.getName().compareTo(o2.getName());
                } else if (o1 == null && o2 == null) {
                    return 0;
                } else if (o1 == null)  {
                    return 1;
                } else {
                    return -1;
                }
            }
        });
        
        // import features
        for ( int i = 0; i < features.length; i++ ) {
            if (features[i] instanceof EAttribute) {
                convertSimpleAttribute(list, features[i]);
            } else if ((features[i] instanceof EReference)
                    && (ModelUtil.hasStereotype(features[i].getEType(), 
                            ModelConstants.STEREOTYPE_PRIMITIVE))) {
                convertSimpleAttribute(list, features[i]);
            }
        }
    }
    
    /**
     * Creates a EAXSDSimpleAttribute for the given attribute and adds it to the
     * given list.
     * 
     * @param list which holds the generated EAXSDSimpleAttribute's
     * @param attribute for which a EAXSDSimpleAttribute will be generated
     */
    private void convertSimpleAttribute(List<EAXSDSimpleAttribute> list,
            EStructuralFeature attribute) {

        EAXSDSimpleAttribute simpleattribute = 
                factory.createEAXSDSimpleAttribute();

        simpleattribute.setAnnotation(convertAnnotation(attribute));
        simpleattribute.setType(createEAXSDType(attribute));
        simpleattribute.setName(getXmlAttributeName(attribute));
        simpleattribute.setCardinality(convertCardinality(attribute));

        if (!simpleattribute.isXmlattribute()) {
            simpleattribute.setType(createEAXSDType(attribute));
        }

        simpleattribute.setRequired(isSimpleAttributeRequired(attribute));

        list.add(simpleattribute);

    }
    
    /**
     * Returns <code>true</code> if min multiplicity is enforced and lower 
     * bound > 0 of the given attribute
     * 
     * @param attribute which will be examined
     * @return <code>true</code> if the given attribute is required
     */
    private boolean isSimpleAttributeRequired(EStructuralFeature attribute) {
        
        TaggedValueHelper helper = new TaggedValueHelper(attribute);
        
        boolean required = helper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_ENFORCE_MIN_MULTIPLICITY, 
                Boolean.FALSE).booleanValue();
        
        return required && attribute.getLowerBound() > 0;
    }
    
    /**
     * Creates a EAXSDCardinality for the given feature
     * 
     * @param feature for which a EAXSDCardinality will be generated
     * @return new EAXSDCardinality for the given feature
     */
    private EAXSDCardinality convertCardinality(EStructuralFeature feature) {
        EAXSDCardinality cardinality = factory.createEAXSDCardinality();
        
        cardinality.setUpperMultiplicity(Integer.valueOf(
                feature.getUpperBound()));
        cardinality.setLowerMultiplicity(Integer.valueOf(
                feature.getLowerBound()));
        
        return cardinality;
    }
    
    /**
     * Generates the schema description for the given EEnum.
     * 
     * @param eEnum The EEnum which will be converted.
     */
    private void convertEnum(EAXSDModel schemaModel, EEnum eEnum) {
        
        EAXSDEnumeration eaXsdEnum = factory.createEAXSDEnumeration();
        eaXsdEnum.setAnnotation(convertAnnotation(eEnum));
        eaXsdEnum.setType(createEAXSDType(eEnum));

        EList<EEnumLiteral> literals = eEnum.getELiterals();
        List<EAXSDEnumerationLiteral> names = new ArrayList<EAXSDEnumerationLiteral>();
        for (EEnumLiteral lit : literals) {
            EAXSDEnumerationLiteral literal = new EAXSDEnumerationLiteral();
            literal.setLiteral(lit.getLiteral());
            literal.setAnnotation(convertAnnotation(lit));
            names.add(literal);
        }
        
        Collections.sort(names, new Comparator<EAXSDEnumerationLiteral>() {

            @Override
            public int compare(EAXSDEnumerationLiteral l1,
                    EAXSDEnumerationLiteral l2) {
                return l1.getLiteral().compareTo(l2.getLiteral());
            }

        });

        for (int i = 0; i < names.size(); i++) {
            eaXsdEnum.getLiteral().add(names.get(i));
        }

        schemaModel.getTypes().add(eaXsdEnum);
    }
    
    /**
     * Creates an EAXSDType for the given classifier
     * 
     * @param classifier for which a EAXSDType will be generated
     * @return new EAXSDType for the given classifier
     */
    private EAXSDType createEAXSDType(EClassifier classifier) {
        
        ClassType clsType = getPrimitiveKind(classifier);
   
        EAXSDType type = factory.createEAXSDType();
   
        type.setXmlname(getXmlElementName(classifier));
        type.setCustom(clsType != ClassType.NATIVE_PRIMITIVE);
        type.setSimple(clsType != ClassType.NOT_PRIMITIVE);
        type.setModelname(classifier.getName());
        type.setInstanceref(isInstanceRef(classifier));
        type.setEmpty(classifier instanceof EEnum);
        type.setPackage(classifier.getEPackage().getName());
        type.setEmpty(false);
        type.setSimplesuffix(false);

        types.add(type);

        return type;
    }
    
    /**
     *  Creates an EAXSDType for the given feature
     * 
     * @param feature for which a EAXSDType will be generated
     * @return new EAXSDType for the given feature
     */
    private EAXSDType createEAXSDType(EStructuralFeature feature) {
        
        EClassifier classifier;
        if (feature instanceof EAttribute) {
            classifier = ((EAttribute) feature).getEAttributeType();
        } else {
            classifier = feature.getEType();
        }

        TaggedValueHelper clsTvHelper = new TaggedValueHelper(classifier);

        String xmlName = clsTvHelper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_XSD_CUSTOM_TYPE);
        
        if (xmlName == null) {
            xmlName = clsTvHelper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_TYPE);
        }
        
        if (xmlName == null) {
            xmlName = clsTvHelper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_NAME);
        }

        ClassType kind = getPrimitiveKind(classifier);
        
        EAXSDType type = factory.createEAXSDType();

        type.setXmlname(xmlName);
        type.setModelname(classifier.getName());
        type.setCustom(kind != ClassType.NATIVE_PRIMITIVE);
        type.setSimple(kind != ClassType.NOT_PRIMITIVE);
        type.setSimplesuffix((kind != ClassType.NATIVE_PRIMITIVE)
                && (isXmlAttribute(feature) || (!hasRoleElement(feature)
                        && !hasRoleWrapperElement(feature)
                        && !hasTypeElement(feature) 
                        && !hasTypeWrapperElement(feature))));
        
        type.setEnum(classifier instanceof EEnum);
        type.setInstanceref(isInstanceRef(classifier));
        type.setPackage(classifier.getEPackage().getName());
        type.setEmpty(false);
        
        types.add(type);
        
        return type;
    }
    
    /**
     * Return the value of xml.attribute
     */
    private boolean isXmlAttribute(EModelElement element) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(element);
        
        return tvHelper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_ATTRIBUTE,
                Boolean.FALSE).booleanValue();
    }
    
    /**
     * Returns <code>true</code> if the given element has the stereotype 
     * ModelConstants.STEREOTYPE_ATPDERIVED or ModelConstants.STEREOTYPE_ATPABSTRACT
     * 
     * @param element
     * @return
     */
    private boolean isDerived(EModelElement element) {
        return ModelUtil.hasStereotype(element, 
                ModelConstants.STEREOTYPE_ATPDERIVED) 
                || ModelUtil.hasStereotype(element, 
                        ModelConstants.STEREOTYPE_ATPABSTRACT);
    }

    /**
     * Returns <code>true</code>, if the given element is a role wrapper element
     * 
     * @param element which will be examined
     * @return <code>true</code>, if the given element is a role wrapper element
     */
    private boolean hasRoleWrapperElement(EModelElement element) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(element);
        
        return tvHelper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_ROLE_WRAPPER_ELEMENT, 
                Boolean.FALSE).booleanValue();
    }

    /**
     * Returns <code>true</code>, if the given element is a type element
     * 
     * @param element which will be examined
     * @return <code>true</code>, if the given element is a type element
     */
    private boolean hasTypeElement(EModelElement element) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(element);
        
        return tvHelper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_TYPE_ELEMENT,
                Boolean.FALSE).booleanValue();
    }

    /**
     * Returns <code>true</code>, if the given element is a role element
     * 
     * @param element which will be examined
     * @return <code>true</code>, if the given element is a role element
     */
    private boolean hasRoleElement(EModelElement element) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(element);
        
        return tvHelper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_ROLE_ELEMENT,
                Boolean.FALSE).booleanValue();
    }

    /**
     * Returns <code>true</code>, if the given element is a type wrapper element
     * 
     * @param element which will be examined
     * @return <code>true</code>, if the given element is a type wrapper element
     */
    private boolean hasTypeWrapperElement(EModelElement element) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(element);
        return tvHelper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_TYPE_WRAPPER_ELEMENT, 
                Boolean.FALSE).booleanValue();
    }
    
    /**
     * Returns <code>true</code> if the given element has the stereotype 
     * <code>ModelConstants.STEREOTYPE_INSTANCE_REF</code>
     * 
     * 
     * @param eElement which will be examined
     * @return <code>true</code> if the given element has the stereotype 
     *         <code>ModelConstants.STEREOTYPE_INSTANCE_REF</code>
     */
    private boolean isInstanceRef(EModelElement eElement) {
        return ModelUtil.hasStereotype(eElement, 
                ModelConstants.STEREOTYPE_INSTANCE_REF);
    }
    
    /**
     * Returns <code>true</code> if the given element has not the stereotype 
     * <code>ModelConstants.STEREOTYPE_ATPMIXED</code> or 
     * <code>ModelConstants.STEREOTYPE_ATPMIXEDSTRING</code>.
     * 
     * 
     * @param eElement which will be examined
     * @return <code>true</code> if the given element has not the stereotype 
     *         <code>ModelConstants.STEREOTYPE_ATPMIXED</code> or 
     *         <code>ModelConstants.STEREOTYPE_ATPMIXEDSTRING</code>
     */
    private boolean isOrdered(EModelElement eElement) {
        
        if (ModelUtil.hasStereotype(eElement,
                ModelConstants.STEREOTYPE_ATPMIXED)
                || ModelUtil.hasStereotype(eElement,
                        ModelConstants.STEREOTYPE_ATPMIXEDSTRING)) {
            return false;
        }
        
        return true;
    }
    
    /**
     * Returns <code>true</code> if the given class is referenced by another 
     * element of the model
     * 
     * @param eClass which will be examined.
     * @return <code>true</code> if the given class is referenced by another 
     *          element of the model
     */
    private boolean isReferenced (EClass eClass) {
        
        Collection <EStructuralFeature.Setting> references;
        Resource resource = eClass.eResource();
        
        if (resource != null) {
            references = UsageCrossReferencer.find(eClass, resource);
        } else {
            //find the root eObject to get the resource
            EObject root = eClass;
            while (root.eContainer() != null) {
                root = root.eContainer();
            }
            
            references = UsageCrossReferencer.find(eClass, root);
        }
        
        for (Setting setting : references) {
            EObject eObject = setting.getEObject();
            if (eObject instanceof EReference) {
                EReference eReference = (EReference) eObject;
                if (!eReference.isContainment()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Returns <code>true</code> if the given element is a global element
     * 
     * @return T<code>true</code> if the given element is a global element
     */
    private boolean isGlobalElement (EModelElement element) {
        TaggedValueHelper helper =  new TaggedValueHelper(element);
        return helper.getBooleanValue(
                ModelConstants.TAGGED_VALUE_XML_GLOBAL_ELEMENT, 
                Boolean.FALSE).booleanValue();
    }
    
    /**
     * Returns plural XML name for given model element.
     * 
     * @param eElement whose plural XML name will be returned
     * @return plural XML name
     */
    private String getXmlElementPluralName(EModelElement eElement) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(eElement);
        
        return tvHelper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_NAME_PLURAL);
    }
    
    /**
     * Returns XML name for given model element.
     * 
     * @param eElement whose XML name will be returned
     * @return XML name
     */
    private String getXmlElementName(EModelElement eElement) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(eElement);
        
        return tvHelper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_NAME);
    }
    
    /**
     * Returns XML attribute name for given model element.
     */
    private String getXmlAttributeName(EModelElement eElement) {
        TaggedValueHelper tvHelper = new TaggedValueHelper(eElement);
        
        return tvHelper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_NAME);
    }
    
    /**
     * Returns the CLASS_TYPE of the given classifier
     * 
     * @param eClassifier Class under examination. Not <code>null</code>.
     * @return the CLASS_TYPE of the given classifier
     */
    private ClassType getPrimitiveKind (EClassifier eClassifier) {

        if (!ModelUtil.hasStereotype(eClassifier, 
                ModelConstants.STEREOTYPE_PRIMITIVE)) {
            return ClassType.NOT_PRIMITIVE;
        }

        TaggedValueHelper helper = new TaggedValueHelper(eClassifier);

        String customtype = helper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_XSD_CUSTOM_TYPE);
        String xsdtype = helper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_XSD_TYPE);

        if ((xsdtype == null) && (customtype == null)) {
            return ClassType.NOT_PRIMITIVE;
        } else if ((customtype != null)) {
            return ClassType.CUSTOM_PRIMITIVE;
        } else {
            return ClassType.NATIVE_PRIMITIVE;
        }
    }
    
    /**
     * Generates the schema description for a EDataType element.
     * 
     * @param schemaModel
     *            The model which buffers the XSD description of the element
     * @param eDataType
     *            The EDataType which will be converted.
     */
    private void convertDataType(EAXSDModel schemaModel, EDataType eDataType) {
        
        TaggedValueHelper helper = new TaggedValueHelper(eDataType);
        String customType = helper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_XSD_CUSTOM_TYPE);
        
        if (customType != null) {

            EAXSDSimpleType simpletype = factory.createEAXSDSimpleType();

            simpletype.setCustomtype(customType);
            simpletype.setAnnotation(convertAnnotation(eDataType));

            String basetype = helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_TYPE, "string");

            simpletype.setBasetype(basetype);

            simpletype.setMaxinclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_INCLUSIVE));
            simpletype.setMaxexclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_EXCLUSIVE));
            
            simpletype.setMininclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_INCLUSIVE));
            simpletype.setMinexclusive(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_EXCLUSIVE));
            
            simpletype.setLen(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_LENGTH));
            simpletype.setMinlength(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MIN_LENGTH));
            simpletype.setMaxlength(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_MAX_LENGTH));
            
            simpletype.setTotaldigits(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_TOTAL_DIGITS));
            simpletype.setFractiondigits(helper.getIntegerValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_FRACTION_DIGITS));
            simpletype.setWhitespace(helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_WHITESPACE));
            simpletype.setPattern(helper.getStringValue(
                    ModelConstants.TAGGED_VALUE_XML_XSD_PATTERN));

            simpletype.setRestrictions((simpletype.getMininclusive() != null)
                    || (simpletype.getMinexclusive() != null)
                    || (simpletype.getWhitespace() != null)
                    || (simpletype.getPattern() != null)
                    || (simpletype.getMaxinclusive() != null)
                    || (simpletype.getLen() != null)
                    || (simpletype.getMinlength() != null)
                    || (simpletype.getMaxlength() != null)
                    || (simpletype.getMaxexclusive() != null)
                    || (simpletype.getTotaldigits() != null)
                    || (simpletype.getFractiondigits() != null));


            schemaModel.getTypes().add(simpletype);
        }
    }
    
    /**
     * Creates a EAXSDAnnotation for the given element
     * 
     * @param eElement
     *            ENamedElement which converts its annotations
     *            
     * @return EAXSDAnnotation for the schema description of the given element
     */
    private EAXSDAnnotation convertAnnotation(ENamedElement eElement) {
        boolean content = false;
        
        EAXSDAnnotation annotation = factory.createEAXSDAnnotation();
        
        String[] stereotypes = ModelUtil.getAllStereotypes(eElement);
        Arrays.sort(stereotypes, new Comparator<String>() {
            @Override
            public int compare(String s0, String s1) {
                if (s0 == null && s1 == null) {
                    return 0;
                } else if (s0 != null && s1 != null) {
                    return s0.compareTo(s1);
                } else if (s0 != null) {
                    return -1;
                } else {
                    //only s1 != null
                    return 1;
                }
            }
        });
        
        for (String stereotype : stereotypes) {
            content = true;
            annotation.getStereotype().add(stereotype);
        }

        Map<String, String> tags = getXMLTags(eElement);
        String[] keys = tags.keySet().toArray(new String[tags.size()]);
        Arrays.sort(keys);
        for (String key : keys) {
            content = true;
            String value = tags.get(key);
            EAXSDTag tag = factory.createEAXSDTag();
            tag.setKey(key);
            tag.setValue(value);
            annotation.getTag().add(tag);
        }
        
        List<EAnnotation> constraints = getConstraints(eElement);

        for (EAnnotation eAnnotation : constraints) {
            EAXSDConstraint constraint = new EAXSDConstraint();
            constraint.setId(eAnnotation.getDetails().get("id"));
            constraint.setLongName(eAnnotation.getDetails().get("long_name"));
            constraint.setBody(eAnnotation.getDetails().get("body"));
            annotation.getConstraints().add(constraint);
        }

        String documentation = ModelUtil.getDocumentation(eElement);
        documentation = removeInvalidChar(documentation);
        if ((documentation != null) && (documentation.trim().length() > 0)) {
            content = true;
            annotation.setDocumentation(documentation.trim());
        }

        if (content) {
            return annotation;
        } else {
            return null;
        }

    }
    
    /**
     * Removes invalid character from the given string
     * 
     * @param s
     * @return
     */
    private String removeInvalidChar(String s) {
        String newString = s;
        
        if (newString != null && newString.length() > 0) { 
            newString = newString.replace("<", "&lt;");
            newString = newString.replace(">", "&gt;");
            newString = newString.replace("&", "&amp;");
        }
        return newString;
    }
    
    /**
     * Returns a List<EAnnotation> of EAnnotation which
     * starts with "Constraint_".
     * 
     * @param eElement
     * @return List<EAnnotation> of EAnnotation
     */
    private List<EAnnotation> getConstraints(ENamedElement eElement) {

        ArrayList<EAnnotation> constraintList = new ArrayList<EAnnotation>();
        
        for (EAnnotation annotation : eElement.getEAnnotations()) {
            if (annotation.getSource().startsWith("Constraint_")) {
                constraintList.add(annotation);
            }
        }
        return constraintList;

    }
    
    /**
     * Determines the value for the entry xml.tags of a TaggedValue Annotation
     * which belongs to the passed model element.
     * 
     * @param eModelElement the model element which holds the value of the
     *            demanded TaggedValue.
     * @return Returns <code>null</code> if the model element does neither have
     *         TaggedValues nor the specific TaggedValue xml.tags, the value of
     *         the entry otherwise.
     */
    private Map<String, String> getXMLTags(EModelElement eElement) {
        TaggedValueHelper helper = new TaggedValueHelper(eElement);

        String tags = helper.getStringValue(
                ModelConstants.TAGGED_VALUE_XML_TAGS);
        
        Map<String, String> tagMap = new Hashtable<String, String>();
        tagMap.clear();

        if (tags != null) {

            String[] pairs = tags.split(";");
            for (int i = 1; i < pairs.length; i++) {
                if (!pairs[i - 1].endsWith("\"")) {
                    pairs[i] = pairs[i - 1] + ";" + pairs[i];
                    pairs[i - 1] = null;
                }
            }

            for (String tagPair : pairs) {
                if (tagPair != null) {
                    int idx = tagPair.indexOf('=');
                    String key = tagPair.substring(0, idx).trim();
                    String value = removeQuotes(tagPair.substring(idx + 1).trim());
                    tagMap.put(key, value);
                }
            }

        }

        return tagMap;
    }
    
    /**
     * Returns the simple content feature. If the given eClass has more than 
     * one feature or the feature should not be represented as simple content
     * null is returned.
     * 
     * @param eClass which will be examined
     * @return the simple content feature
     */
    private EStructuralFeature getSimpleContentFeature(EClass eClass) {
        
        List<EStructuralFeature> features = getFeatures(eClass,
                XMLMappingKind.GENERATE_ELEMENT);
        
        if (features.size() == 1) {
            EStructuralFeature feature = features.get(0);
            EClassifier eType= feature.getEType();
            if (feature.getUpperBound() == 1 
                    && !hasTypeElement(feature)
                    && !hasRoleElement(feature)
                    && !hasTypeWrapperElement(feature)
                    && !hasRoleWrapperElement(feature)
                    && (eType instanceof EDataType)) {
                return feature;
            }
        }

        return null;
    }
    
    /**
     * Removes the leading and trailing quote from the given string
     * 
     * @param str
     * @return a string without a leading and a trailing quote
     */
    private String removeQuotes(String str) {
        if (str.length() >= 2 
                && (str.charAt(0) == '\"')
                && (str.charAt(str.length() - 1) == '\"')) {
            str = str.substring(1, str.length() - 1);
        }
        return str;
    }

    /**
     * Imports the necessary content from the supplied model into the schema
     * 
     * @param model
     *            The model which will be converted.
     * @return The representation for the schema.
     */
    public EAXSDModel convert(EPackage model, String version) {

        classTypes.clear();
        types.clear();

        EAXSDModel result = factory.createEAXSDModel();
        result.setAdministrative(createAdministrativeData(model, version));

        convertModel(result, model);

        for (EAXSDType reftype : types) {
            String xmlName = reftype.getXmlname();
            CustomClassType type = null;
            if (xmlName != null) {
                type = classTypes.get(reftype.getXmlname());
            }
            if (type != null) {
                reftype.setEmpty(type.isEmpty());
            }
        }

        return result;
    }
    
    /**
     * Creates a new administrative data.
     * 
     * @param model
     *            The model package used to access administrative information.
     * @return administrative data
     */
    private EAXSDAdministrativeData createAdministrativeData(EPackage model, 
            String version) {
        ModelInformation modelinfo = ModelInformationFactory.create(model, 
                version);
        EAXSDAdministrativeData res = factory.createEAXSDAdministrativeData();
        res.setDate(new Date());
        res.setNamespaceuri(modelinfo.getNSUri());
        res.setNamespaceprefix(modelinfo.getNSPrefix());
        
        if (modelinfo.getNSPrefix() == null) {
            res.setNamespaceprefix(ModelConstants.DEFAULT_NSPREFIX);
        }
        
        return res;
    }
}
