/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EPackage;

import eaadapter.common.logging.Logger;
import eaadapter.common.logging.LoggerImpl;
import eaadapter.xsd.common.MetamodelVersion;
import eaadapter.xsd.model.EAXSDAdministrativeData;
import eaadapter.xsd.model.EAXSDAttribute;
import eaadapter.xsd.model.EAXSDClassType;
import eaadapter.xsd.model.EAXSDDataType;
import eaadapter.xsd.model.EAXSDDerived;
import eaadapter.xsd.model.EAXSDEnumeration;
import eaadapter.xsd.model.EAXSDModel;
import eaadapter.xsd.model.EAXSDReference;
import eaadapter.xsd.model.EAXSDSimpleAttribute;
import eaadapter.xsd.model.EAXSDSimpleType;
import eaadapter.xsd.output.StringTemplateController;
import eaadapter.xsd.util.IOUtil;

public class XSDGenerator {
    
    private static Logger logger = new LoggerImpl();
    
    private File outputFile = null;
    private EPackage eastadlModel = null;
    private String version = null;
    
    public XSDGenerator(File output, EPackage eastadlModel, String version) {
        this.outputFile = output;
        this.eastadlModel = eastadlModel;
        this.version = version;
    }
    
    public String generateContent(StringTemplateController stController,
            EAXSDModel model) {
        
        EAXSDAdministrativeData adminData = model.getAdministrative();
        
        String disclaimer = stController.getDisclaimerXSDString(
                model.getAdministrative().getDisclaimer());
        
        List<String> elements = new ArrayList<String>();

        for (int i = 0; i < model.getTypes().size(); i++) {
            EAXSDDataType type = model.getTypes().get(i);
            
            if (type instanceof EAXSDClassType) {
                elements.add(generateClassType(stController,
                        (EAXSDClassType) type));
            } else if (type instanceof EAXSDEnumeration) {
                elements.add(stController.getEnumerationXSDString(
                        (EAXSDEnumeration) type));
            } else {
                elements.add(generateSimpleType(stController,
                        (EAXSDSimpleType) type));
            }
        }

        return stController.getSkeletonXSDString(adminData, disclaimer, elements);
    }

    private String generateClassType(StringTemplateController stController,
            EAXSDClassType classType) {
        
        String group = null;
        List<String> simpleAttributes = generateSimpleAttributes(stController,
                classType.getSimpleattribute());
        
        if (simpleAttributes != null && simpleAttributes.size() > 0) {
            group = stController.getAttributeGroupXSDString(classType, 
                    simpleAttributes);
        }
        
        stController.setMinMax(classType.isOrdered());
        
        
        String subTypes = null;
        if (classType.getSubtypes() != null) {
            subTypes = stController.getSubTypesXSDString(classType
                    .getSubtypes());
        }
        
        String abstractType = null;
        if (classType.getAbstracttype() != null) {
            abstractType = stController.getAbstractTypeXSDString(classType
                    .getAbstracttype());
        }
        
        String global = null;
        if (classType.getGlobaltype() != null) {
            global = stController.getGlobalXSDString(classType.getGlobaltype());
        }
        
        List<String> elementAttributes = generateElementAttributes(stController,
                classType.getElementattributes());
        
        String result = stController.getClassTypeXSDString(classType, group, 
                elementAttributes, abstractType, subTypes, global);
        
        stController.setMinMax(true);
        
        return result;
    }

    private List<String> generateElementAttributes(
            StringTemplateController stController,
            List<EAXSDDataType> attributes) {
        
        if (attributes.size() <= 0) {
            return null;
        }
        
        List<String> result = new ArrayList<String>();
        
        for (EAXSDDataType attribute : attributes) {
            String ele = generateElementAttribute(stController, attribute);
            
            if (ele != null) {
                result.add(ele);
            }
        }
        
        return result;
    }

    private String generateElementAttribute(
            StringTemplateController stController, EAXSDDataType attribute) {

        String name = attribute.getClass().getSimpleName();
        if (attribute instanceof EAXSDAttribute) {
            return stController.getAttributeXSDString((EAXSDAttribute) attribute);
        } else if (attribute instanceof EAXSDReference) {
            return stController.getReferenceXSDString((EAXSDReference) attribute);
        } else if (attribute instanceof EAXSDDerived) {
            return stController.getDerivedXSDString((EAXSDDerived) attribute);
        } else {
            int caseno = Integer.parseInt(name.substring(name.length() - 2));
            return stController.getElementCaseXSDString(caseno, attribute);
        }
    }

    private List<String> generateSimpleAttributes(
            StringTemplateController stController,
            List<EAXSDSimpleAttribute> attributes) {
        
        if (attributes.size() <= 0) {
            return null;
        }
        
        List<String> result = new ArrayList<String>();
        
        for (EAXSDSimpleAttribute attribute : attributes) {
            result.add(stController.getSimpleAttributeXSDString(attribute));
        }
        
        return result;
    }

    private String generateSimpleType(StringTemplateController stController,
            EAXSDSimpleType simpleType) {
        
        List<String> simpleAttributes = generateSimpleAttributes(stController,
                simpleType.getSimpleattribute());
        
        String group = null;
        if (simpleAttributes != null) {
            group = stController.getAttributeGroupXSDString(simpleType, 
                    simpleAttributes);
        }

        return stController.getSimpleTypeXSDString(simpleType, group);
    }
    
    private String generateString(EAXSDModel model) {
        
        String templateDir = getStringTemplateDir();
        if (templateDir == null) {
            MetamodelVersion mv = MetamodelVersion.getNewInstance(version);
            logger.error("Can't find template directory for the version "
                    + mv.toString() + " of the meta model!");
            return null;
        }
        
        StringTemplateController stController = new StringTemplateController(
                model, templateDir);
        
        return generateContent(stController, model);
    }
    
    /**
     * Tries to determine the template directory which belongs to the version
     * of the meta model 
     * 
     * @return template directory path
     */
    private String getStringTemplateDir() {
        MetamodelVersion mv = MetamodelVersion.getNewInstance(version);
        
        String path = "/resources/templates/" + mv.toString();
        String templateDir = null;
        
        String tmpDir = path + "." + mv.getRevision();
        if (Activator.getDefault().getResourceURL(tmpDir) != null) {
            templateDir = tmpDir;
        }
        
        tmpDir = path;
        if (templateDir == null 
                && Activator.getDefault().getResourceURL(tmpDir) != null) {
            templateDir = tmpDir;
        }
        
        if (templateDir != null) {
            templateDir = templateDir + "/xsd";
        }
        
        return templateDir;
    }
    
    public void run() {
        if (outputFile == null) {
            return;
        }
        
        Ecore2XSDModel ecore2XSDModel = new Ecore2XSDModel();
        EAXSDModel model = ecore2XSDModel.convert(eastadlModel, version);

        String content = generateString(model);
        
        IOUtil.delete(outputFile);
        
        try {
            IOUtil.write(outputFile, content);
        } catch (IOException e) {
            logger.error("Error while creating output file " 
                    + outputFile.getName() + "\n" + e.getLocalizedMessage());
        }
    }
}
