/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd.common;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MetamodelVersion implements Comparable<MetamodelVersion> {
    
    private static final String STR_UNSPECIFIED = "UNSPECIFIED";

    public static final MetamodelVersion UNSPECIFIED = new MetamodelVersion();

    /**
     * Regular expression matching all allowed version numbers
     */
    private static final String VERSION_PATTERN = "(\\d+)(?:\\.(\\d+))?(?:\\.(\\d+))?";

    private Double numericValue;
    
    private Integer major;
    private Integer minor;
    private Integer revision;

    /**
     * Private default constructor
     */
    private MetamodelVersion() {
        numericValue = Double.valueOf(0);
        major = Integer.valueOf(-1);
        minor = Integer.valueOf(-1);
        revision = Integer.valueOf(-1);
    }

    /**
     * Initialises this version from the supplied string. A version is supposed
     * to consist at least of a major one digit version number. Additionally a
     * minor version number and a revision number might be provided, each
     * expected to be exactly one digit, seperated from its predecessor by a
     * dot.
     * 
     * @param str
     *            The version string containing the version information. Not
     *            <code>null</code>.
     */
    private MetamodelVersion(String str) {
        this();

        Pattern versionPattern = Pattern.compile(VERSION_PATTERN);

        Matcher matcher = versionPattern.matcher(str);
        if (matcher.matches()) {

            major = Integer.valueOf(matcher.group(1));

            if (matcher.group(2) != null) {
                minor = Integer.valueOf(matcher.group(2));
            }

            if (matcher.group(3) != null) {
                revision = Integer.valueOf(matcher.group(3));
            }

        }

        if (!isUnspecified()) {
            if (minor.intValue() == -1) {
                numericValue = Double.valueOf(major.doubleValue());
            } else {
                numericValue = Double.valueOf(major.doubleValue()
                        + minor.doubleValue() / 100);
            }
        }

    }
    
    /**
     * Returns the major version.
     * 
     * @return major version.
     */
    public int getMajor() {
        return major.intValue();
    }

    /**
     * Returns the minor version.
     * 
     * @return minor version.
     */
    public int getMinor() {
        return minor.intValue();
    }

    /**
     * Returns <code>true</code> if this instance provides an unspecified
     * information.
     * 
     * @return <code>true</code> <=> This instance provides an unspecified
     *         information.
     */
    public boolean isUnspecified() {
        return major.intValue() == -1;
    }

    /**
     * Returns the current revision associated with this object.
     * 
     * @return revision number. Maybe -1.
     */
    public int getRevision() {
        return revision.intValue();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        return numericValue.hashCode();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof MetamodelVersion) {
            MetamodelVersion other = (MetamodelVersion) obj;
            return numericValue.equals(other.numericValue);
        } else {
            return false;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        if (major.intValue() == -1) {
            return STR_UNSPECIFIED;
        } else {
            if (minor.intValue() == -1) {
                return String.valueOf(major);
            } else {
                return String.format("%d.%d", major, minor);
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int compareTo(MetamodelVersion mmv) {
        if (mmv == null) {
            return -1;
        }
        return mmv.numericValue.compareTo(numericValue);
    }

    /**
     * Returns an instance of the MetamodelVersion. If the string is not 
     * <code>null</code> it can be formatted as followed: 
     * <li><code>%d.%d.%d</code></li>
     * <li><code>%d.%d</code></li>
     * <li><code>%d</code></li>
     * 
     * @param versionString
     *            The string providing the version information. Maybe
     *            <code>null</code>.
     * @return The MetamodelVersion if the supplied version string was valid.
     */
    public static final MetamodelVersion getNewInstance(String versionString) {
        if (versionString != null) {
            try {
                return new MetamodelVersion(versionString);
            } catch (NumberFormatException ex) {
                return null;
            }
        }
        return null;
    }
}
